"""Play queue management."""

import random
from dataclasses import dataclass, field
from typing import Optional

from wrkmon.core.youtube import SearchResult


@dataclass
class QueueItem:
    """An item in the play queue."""

    video_id: str
    title: str
    channel: str
    duration: int
    added_at: float = 0.0
    playback_position: int = 0  # Last played position in seconds

    @classmethod
    def from_search_result(cls, result: SearchResult) -> "QueueItem":
        """Create a queue item from a search result."""
        import time

        return cls(
            video_id=result.video_id,
            title=result.title,
            channel=result.channel,
            duration=result.duration,
            added_at=time.time(),
            playback_position=0,
        )

    @classmethod
    def from_dict(cls, data: dict) -> "QueueItem":
        """Create a queue item from a dictionary."""
        import time

        return cls(
            video_id=data["video_id"],
            title=data["title"],
            channel=data["channel"],
            duration=data["duration"],
            added_at=data.get("added_at", time.time()),
            playback_position=data.get("playback_position", 0),
        )

    def to_dict(self) -> dict:
        """Convert to dictionary for serialization."""
        return {
            "video_id": self.video_id,
            "title": self.title,
            "channel": self.channel,
            "duration": self.duration,
            "added_at": self.added_at,
            "playback_position": self.playback_position,
        }

    @property
    def url(self) -> str:
        """Get the YouTube URL."""
        return f"https://www.youtube.com/watch?v={self.video_id}"


@dataclass
class PlayQueue:
    """Manages the play queue."""

    items: list[QueueItem] = field(default_factory=list)
    current_index: int = -1
    shuffle_mode: bool = False
    repeat_mode: str = "none"  # none, one, all
    _shuffle_order: list[int] = field(default_factory=list)

    @property
    def current(self) -> Optional[QueueItem]:
        """Get the current item."""
        if 0 <= self.current_index < len(self.items):
            if self.shuffle_mode and self._shuffle_order:
                actual_index = self._shuffle_order[self.current_index]
                return self.items[actual_index]
            return self.items[self.current_index]
        return None

    @property
    def is_empty(self) -> bool:
        """Check if queue is empty."""
        return len(self.items) == 0

    @property
    def length(self) -> int:
        """Get queue length."""
        return len(self.items)

    @property
    def has_next(self) -> bool:
        """Check if there's a next item."""
        if self.repeat_mode == "all":
            return len(self.items) > 0
        if self.repeat_mode == "one":
            return self.current is not None
        return self.current_index < len(self.items) - 1

    @property
    def has_previous(self) -> bool:
        """Check if there's a previous item."""
        if self.repeat_mode in ("all", "one"):
            return len(self.items) > 0
        return self.current_index > 0

    def add(self, item: QueueItem) -> int:
        """Add an item to the queue. Returns position."""
        self.items.append(item)
        if self.shuffle_mode:
            self._shuffle_order.append(len(self.items) - 1)
        return len(self.items) - 1

    def add_search_result(self, result: SearchResult) -> int:
        """Add a search result to the queue."""
        return self.add(QueueItem.from_search_result(result))

    def add_multiple(self, items: list[QueueItem]) -> None:
        """Add multiple items to the queue."""
        for item in items:
            self.add(item)

    def remove(self, index: int) -> Optional[QueueItem]:
        """Remove an item by index."""
        if 0 <= index < len(self.items):
            item = self.items.pop(index)

            # Update shuffle order
            if self.shuffle_mode:
                self._shuffle_order = [i for i in self._shuffle_order if i != index]
                self._shuffle_order = [i - 1 if i > index else i for i in self._shuffle_order]

            # Update current index
            if index < self.current_index:
                self.current_index -= 1
            elif index == self.current_index:
                # Current item was removed
                if self.current_index >= len(self.items):
                    self.current_index = len(self.items) - 1

            return item
        return None

    def clear(self) -> None:
        """Clear the queue."""
        self.items.clear()
        self._shuffle_order.clear()
        self.current_index = -1

    def next(self) -> Optional[QueueItem]:
        """Move to and return the next item."""
        if not self.items:
            return None

        if self.repeat_mode == "one":
            return self.current

        if self.shuffle_mode:
            if self.current_index < len(self._shuffle_order) - 1:
                self.current_index += 1
            elif self.repeat_mode == "all":
                self.current_index = 0
            else:
                return None
        else:
            if self.current_index < len(self.items) - 1:
                self.current_index += 1
            elif self.repeat_mode == "all":
                self.current_index = 0
            else:
                return None

        return self.current

    def previous(self) -> Optional[QueueItem]:
        """Move to and return the previous item."""
        if not self.items:
            return None

        if self.repeat_mode == "one":
            return self.current

        if self.current_index > 0:
            self.current_index -= 1
        elif self.repeat_mode == "all":
            self.current_index = len(self.items) - 1
        else:
            return None

        return self.current

    def jump_to(self, index: int) -> Optional[QueueItem]:
        """Jump to a specific index."""
        if 0 <= index < len(self.items):
            self.current_index = index
            return self.current
        return None

    def shuffle(self) -> None:
        """Enable shuffle mode and create shuffle order."""
        self.shuffle_mode = True
        self._create_shuffle_order()

    def unshuffle(self) -> None:
        """Disable shuffle mode."""
        self.shuffle_mode = False
        # Update current index to actual position
        if self._shuffle_order and 0 <= self.current_index < len(self._shuffle_order):
            self.current_index = self._shuffle_order[self.current_index]
        self._shuffle_order.clear()

    def toggle_shuffle(self) -> bool:
        """Toggle shuffle mode. Returns new state."""
        if self.shuffle_mode:
            self.unshuffle()
        else:
            self.shuffle()
        return self.shuffle_mode

    def _create_shuffle_order(self) -> None:
        """Create a new shuffle order."""
        self._shuffle_order = list(range(len(self.items)))
        random.shuffle(self._shuffle_order)

        # Move current item to front if playing
        if self.current_index >= 0:
            current_actual = self.current_index
            if current_actual in self._shuffle_order:
                self._shuffle_order.remove(current_actual)
                self._shuffle_order.insert(0, current_actual)
                self.current_index = 0

    def set_repeat(self, mode: str) -> None:
        """Set repeat mode: none, one, all."""
        if mode in ("none", "one", "all"):
            self.repeat_mode = mode

    def cycle_repeat(self) -> str:
        """Cycle through repeat modes. Returns new mode."""
        modes = ["none", "one", "all"]
        current_idx = modes.index(self.repeat_mode)
        self.repeat_mode = modes[(current_idx + 1) % len(modes)]
        return self.repeat_mode

    def move(self, from_index: int, to_index: int) -> bool:
        """Move an item from one position to another."""
        if not (0 <= from_index < len(self.items) and 0 <= to_index < len(self.items)):
            return False

        item = self.items.pop(from_index)
        self.items.insert(to_index, item)

        # Update current index
        if self.current_index == from_index:
            self.current_index = to_index
        elif from_index < self.current_index <= to_index:
            self.current_index -= 1
        elif to_index <= self.current_index < from_index:
            self.current_index += 1

        # Recreate shuffle order if needed
        if self.shuffle_mode:
            self._create_shuffle_order()

        return True

    def get_upcoming(self, count: int = 5) -> list[QueueItem]:
        """Get upcoming items in queue."""
        result = []
        start = self.current_index + 1

        if self.shuffle_mode and self._shuffle_order:
            for i in range(start, min(start + count, len(self._shuffle_order))):
                actual_idx = self._shuffle_order[i]
                result.append(self.items[actual_idx])
        else:
            for i in range(start, min(start + count, len(self.items))):
                result.append(self.items[i])

        return result

    def to_list(self) -> list[QueueItem]:
        """Get all items in current play order."""
        if self.shuffle_mode and self._shuffle_order:
            return [self.items[i] for i in self._shuffle_order]
        return list(self.items)

    def update_playback_position(self, video_id: str, position: int) -> None:
        """Update the playback position for a queue item."""
        for item in self.items:
            if item.video_id == video_id:
                item.playback_position = position
                break

    def get_playback_position(self, video_id: str) -> int:
        """Get the saved playback position for a queue item."""
        for item in self.items:
            if item.video_id == video_id:
                return item.playback_position
        return 0

    def to_dict_list(self) -> list[dict]:
        """Convert all items to list of dicts for serialization."""
        return [item.to_dict() for item in self.items]

    def load_from_dicts(self, items: list[dict], current_index: int = -1) -> None:
        """Load queue from list of dicts."""
        self.clear()
        for data in items:
            item = QueueItem.from_dict(data)
            self.items.append(item)
        self.current_index = current_index if -1 <= current_index < len(self.items) else -1
        if self.shuffle_mode:
            self._create_shuffle_order()



"""Custom Textual messages for wrkmon component communication."""

from textual.message import Message

from wrkmon.core.youtube import SearchResult


class TrackSelected(Message):
    """Emitted when a track is selected for playback."""

    def __init__(self, result: SearchResult) -> None:
        self.result = result
        super().__init__()


class TrackQueued(Message):
    """Emitted when a track is added to queue."""

    def __init__(self, result: SearchResult) -> None:
        self.result = result
        super().__init__()


class PlaybackStateChanged(Message):
    """Emitted when playback state changes."""

    def __init__(
        self,
        is_playing: bool,
        position: float = 0.0,
        duration: float = 0.0,
        title: str = "",
    ) -> None:
        self.is_playing = is_playing
        self.position = position
        self.duration = duration
        self.title = title
        super().__init__()


class VolumeChanged(Message):
    """Emitted when volume changes."""

    def __init__(self, volume: int) -> None:
        self.volume = volume
        super().__init__()


class ViewChanged(Message):
    """Emitted when the active view should change."""

    def __init__(self, view_name: str) -> None:
        self.view_name = view_name
        super().__init__()


class SearchStarted(Message):
    """Emitted when a search begins."""

    def __init__(self, query: str) -> None:
        self.query = query
        super().__init__()


class SearchCompleted(Message):
    """Emitted when search results are ready."""

    def __init__(self, results: list[SearchResult], query: str) -> None:
        self.results = results
        self.query = query
        super().__init__()


class QueueUpdated(Message):
    """Emitted when the queue changes."""

    def __init__(self, queue_length: int, current_index: int) -> None:
        self.queue_length = queue_length
        self.current_index = current_index
        super().__init__()


class StatusMessage(Message):
    """Emitted to show a status message to the user."""

    def __init__(self, message: str, level: str = "info") -> None:
        self.message = message
        self.level = level  # "info", "success", "warning", "error"
        super().__init__()



"""Version checker and updater for wrkmon."""

import asyncio
import json
import logging
import shutil
import subprocess
import sys
import urllib.request
from dataclasses import dataclass
from pathlib import Path
from typing import Optional
from packaging import version

from wrkmon import __version__

logger = logging.getLogger("wrkmon.updater")

PYPI_URL = "https://pypi.org/pypi/wrkmon/json"
CHECK_INTERVAL_HOURS = 24


@dataclass
class UpdateInfo:
    """Information about an available update."""

    current_version: str
    latest_version: str
    is_update_available: bool
    release_url: str = "https://pypi.org/project/wrkmon/"

    @property
    def update_command(self) -> str:
        """Get the command to update wrkmon."""
        return "pip install --upgrade wrkmon"


def get_current_version() -> str:
    """Get the current installed version."""
    return __version__


def check_pypi_version() -> Optional[str]:
    """
    Check PyPI for the latest version of wrkmon.

    Returns:
        The latest version string, or None if check failed.
    """
    try:
        req = urllib.request.Request(
            PYPI_URL,
            headers={"Accept": "application/json", "User-Agent": "wrkmon-updater"}
        )
        with urllib.request.urlopen(req, timeout=5) as response:
            data = json.loads(response.read().decode())
            return data.get("info", {}).get("version")
    except Exception as e:
        logger.debug(f"Failed to check PyPI version: {e}")
        return None


async def check_pypi_version_async() -> Optional[str]:
    """Async wrapper for checking PyPI version."""
    return await asyncio.to_thread(check_pypi_version)


def compare_versions(current: str, latest: str) -> bool:
    """
    Check if an update is available.

    Returns:
        True if latest > current (update available).
    """
    try:
        return version.parse(latest) > version.parse(current)
    except Exception:
        # Fallback to string comparison
        return latest != current and latest > current


def check_for_updates() -> Optional[UpdateInfo]:
    """
    Check if a newer version is available on PyPI.

    Returns:
        UpdateInfo if check succeeded, None if check failed.
    """
    current = get_current_version()
    latest = check_pypi_version()

    if latest is None:
        return None

    return UpdateInfo(
        current_version=current,
        latest_version=latest,
        is_update_available=compare_versions(current, latest),
    )


async def check_for_updates_async() -> Optional[UpdateInfo]:
    """Async version of check_for_updates."""
    return await asyncio.to_thread(check_for_updates)


def perform_update() -> tuple[bool, str]:
    """
    Attempt to update wrkmon using pip.

    Returns:
        tuple: (success, message)
    """
    try:
        result = subprocess.run(
            [sys.executable, "-m", "pip", "install", "--upgrade", "wrkmon"],
            capture_output=True,
            text=True,
            timeout=120,
        )

        if result.returncode == 0:
            return True, "Update successful! Please restart wrkmon."
        else:
            return False, f"Update failed: {result.stderr}"
    except subprocess.TimeoutExpired:
        return False, "Update timed out. Try manually: pip install --upgrade wrkmon"
    except Exception as e:
        return False, f"Update error: {e}"


async def perform_update_async() -> tuple[bool, str]:
    """Async version of perform_update."""
    return await asyncio.to_thread(perform_update)


# ============================================================
# Dependency checkers for better compatibility
# ============================================================

def is_deno_installed() -> bool:
    """Check if deno is installed."""
    return shutil.which("deno") is not None


def is_nodejs_installed() -> bool:
    """Check if Node.js is installed."""
    return shutil.which("node") is not None


def get_js_runtime() -> Optional[str]:
    """Get the available JavaScript runtime for yt-dlp."""
    if is_deno_installed():
        return "deno"
    if is_nodejs_installed():
        return "nodejs"
    return None


def get_deno_install_command() -> str:
    """Get the command to install deno for the current platform."""
    if sys.platform == "win32":
        return "irm https://deno.land/install.ps1 | iex"
    else:
        return "curl -fsSL https://deno.land/install.sh | sh"


def install_deno() -> tuple[bool, str]:
    """
    Attempt to install deno.

    Returns:
        tuple: (success, message)
    """
    if is_deno_installed():
        return True, "deno is already installed"

    try:
        if sys.platform == "win32":
            # Try winget first
            try:
                result = subprocess.run(
                    ["winget", "install", "--id", "DenoLand.Deno", "-e", "--silent"],
                    capture_output=True,
                    timeout=300,
                )
                if result.returncode == 0:
                    return True, "deno installed via winget"
            except Exception:
                pass

            # Try scoop
            try:
                result = subprocess.run(
                    ["scoop", "install", "deno"],
                    capture_output=True,
                    timeout=300,
                )
                if result.returncode == 0:
                    return True, "deno installed via scoop"
            except Exception:
                pass

            # Try chocolatey
            try:
                result = subprocess.run(
                    ["choco", "install", "deno", "-y"],
                    capture_output=True,
                    timeout=300,
                )
                if result.returncode == 0:
                    return True, "deno installed via chocolatey"
            except Exception:
                pass

            return False, f"Please install deno manually:\n{get_deno_install_command()}"

        else:
            # Unix - try package managers first
            if sys.platform == "darwin":
                # macOS - try brew
                try:
                    result = subprocess.run(
                        ["brew", "install", "deno"],
                        capture_output=True,
                        timeout=300,
                    )
                    if result.returncode == 0:
                        return True, "deno installed via homebrew"
                except Exception:
                    pass
            else:
                # Linux - try snap
                try:
                    result = subprocess.run(
                        ["snap", "install", "deno"],
                        capture_output=True,
                        timeout=300,
                    )
                    if result.returncode == 0:
                        return True, "deno installed via snap"
                except Exception:
                    pass

            # Try the official install script
            try:
                result = subprocess.run(
                    ["sh", "-c", "curl -fsSL https://deno.land/install.sh | sh"],
                    capture_output=True,
                    timeout=300,
                )
                if result.returncode == 0:
                    return True, "deno installed via official script"
            except Exception:
                pass

            return False, f"Please install deno manually:\n{get_deno_install_command()}"

    except Exception as e:
        return False, f"Installation failed: {e}"


async def install_deno_async() -> tuple[bool, str]:
    """Async version of install_deno."""
    return await asyncio.to_thread(install_deno)


def check_dependencies() -> dict:
    """
    Check all optional dependencies for optimal functionality.

    Returns:
        dict with dependency status.
    """
    from wrkmon.utils.mpv_installer import is_mpv_installed

    return {
        "mpv": {
            "installed": is_mpv_installed(),
            "required": True,
            "description": "Media player for audio playback",
        },
        "deno": {
            "installed": is_deno_installed(),
            "required": False,
            "description": "JavaScript runtime for better YouTube compatibility",
        },
        "nodejs": {
            "installed": is_nodejs_installed(),
            "required": False,
            "description": "Alternative JavaScript runtime",
        },
        "js_runtime": {
            "installed": get_js_runtime() is not None,
            "required": False,
            "description": "JavaScript runtime (deno or nodejs) for full YouTube support",
        },
    }


def get_missing_dependencies() -> list[str]:
    """Get list of missing recommended dependencies."""
    deps = check_dependencies()
    missing = []

    if not deps["mpv"]["installed"]:
        missing.append("mpv")
    if not deps["js_runtime"]["installed"]:
        missing.append("deno (recommended for YouTube)")

    return missing



"""Audio player using mpv with IPC for proper control."""

import asyncio
import json
import logging
import os
import subprocess
import sys
import time
from typing import Optional

from wrkmon.utils.config import get_config

logger = logging.getLogger("wrkmon.player")


class AudioPlayer:
    """mpv audio player with IPC for pause/resume/seek."""

    def __init__(self):
        self._config = get_config()
        self._process: Optional[subprocess.Popen] = None
        self._current_url: Optional[str] = None
        self._volume = 80
        self._paused = False
        self._position = 0.0
        self._duration = 0.0
        self._pipe_path = r"\\.\pipe\wrkmon_mpv" if sys.platform == "win32" else "/tmp/wrkmon_mpv.sock"
        self._pipe = None

    @property
    def is_connected(self) -> bool:
        return self._process is not None and self._process.poll() is None

    @property
    def is_playing(self) -> bool:
        return self.is_connected and not self._paused

    @property
    def current_position(self) -> float:
        return self._position

    @property
    def duration(self) -> float:
        return self._duration

    @property
    def volume(self) -> int:
        return self._volume

    async def start(self) -> bool:
        """Initialize player (no-op, starts on play)."""
        return True

    def _send_command(self, command: list) -> Optional[dict]:
        """Send command to mpv via IPC and get response."""
        if not self.is_connected:
            return None

        try:
            if sys.platform == "win32":
                import win32file
                import win32pipe

                # Open pipe
                handle = win32file.CreateFile(
                    self._pipe_path,
                    win32file.GENERIC_READ | win32file.GENERIC_WRITE,
                    0, None,
                    win32file.OPEN_EXISTING,
                    0, None
                )

                # Send command
                cmd = json.dumps({"command": command}) + "\n"
                win32file.WriteFile(handle, cmd.encode())

                # Read response
                result, data = win32file.ReadFile(handle, 4096)
                win32file.CloseHandle(handle)

                if data:
                    return json.loads(data.decode().strip())
            else:
                # Unix socket
                import socket
                sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
                sock.connect(self._pipe_path)
                sock.settimeout(1.0)

                cmd = json.dumps({"command": command}) + "\n"
                sock.send(cmd.encode())

                data = sock.recv(4096)
                sock.close()

                if data:
                    return json.loads(data.decode().strip())

        except Exception as e:
            logger.debug(f"IPC command failed: {e}")

        return None

    async def _send_command_async(self, command: list) -> Optional[dict]:
        """Async wrapper for send_command."""
        return await asyncio.to_thread(self._send_command, command)

    async def play(self, url: str) -> bool:
        """Play audio from URL - stealth mode with IPC."""
        logger.info(f"=== player.play() called ===")
        logger.info(f"  URL: {url[:100]}...")

        # Kill any existing playback
        await self.stop()

        mpv_path = self._config.mpv_path
        logger.info(f"  mpv_path: {mpv_path}")

        # Check if mpv exists
        mpv_exists = os.path.exists(mpv_path) if mpv_path != "mpv" else True
        logger.info(f"  mpv exists: {mpv_exists}")

        # Args with IPC server for control
        args = [
            mpv_path,
            "--no-video",
            "--no-terminal",
            "--really-quiet",
            "--no-osc",
            "--no-osd-bar",
            "--force-window=no",
            "--audio-display=no",
            f"--volume={self._volume}",
            f"--input-ipc-server={self._pipe_path}",
            url,
        ]
        logger.info(f"  IPC pipe: {self._pipe_path}")

        try:
            if sys.platform == "win32":
                logger.info("  Creating Windows subprocess...")
                self._process = subprocess.Popen(
                    args,
                    stdin=subprocess.DEVNULL,
                    stdout=subprocess.DEVNULL,
                    stderr=subprocess.DEVNULL,
                    creationflags=subprocess.CREATE_NO_WINDOW,
                )
            else:
                logger.info("  Creating Unix subprocess...")
                self._process = subprocess.Popen(
                    args,
                    stdin=subprocess.DEVNULL,
                    stdout=subprocess.DEVNULL,
                    stderr=subprocess.DEVNULL,
                    start_new_session=True,
                )

            logger.info(f"  Process created, PID: {self._process.pid}")
            self._current_url = url
            self._paused = False

            # Wait for mpv to start and create pipe
            await asyncio.sleep(1.0)

            poll_result = self._process.poll()
            if poll_result is None:
                logger.info("  SUCCESS - mpv is running!")
                # Try to get duration
                await self._update_properties()
                return True
            else:
                logger.error(f"  FAILED - mpv exited with code: {poll_result}")
                return False

        except Exception as e:
            logger.exception(f"  EXCEPTION: {e}")
            return False

    async def _update_properties(self) -> None:
        """Update position and duration from mpv."""
        try:
            # Get time position
            result = await self._send_command_async(["get_property", "time-pos"])
            if result and "data" in result:
                self._position = float(result["data"] or 0)

            # Get duration
            result = await self._send_command_async(["get_property", "duration"])
            if result and "data" in result:
                self._duration = float(result["data"] or 0)

            # Get pause state
            result = await self._send_command_async(["get_property", "pause"])
            if result and "data" in result:
                self._paused = bool(result["data"])

        except Exception as e:
            logger.debug(f"Failed to update properties: {e}")

    async def stop(self) -> None:
        """Stop playback."""
        if self._process:
            try:
                # Try graceful quit first
                self._send_command(["quit"])
                await asyncio.sleep(0.2)
            except Exception:
                pass

            try:
                self._process.terminate()
                self._process.wait(timeout=2)
            except Exception:
                try:
                    self._process.kill()
                except Exception:
                    pass
            self._process = None

        self._paused = False
        self._position = 0.0
        self._duration = 0.0

    async def pause(self) -> None:
        """Pause playback via IPC."""
        if self.is_connected:
            result = await self._send_command_async(["set_property", "pause", True])
            if result:
                self._paused = True
                logger.info("Paused via IPC")
            else:
                # Fallback - just set flag
                self._paused = True

    async def resume(self) -> None:
        """Resume playback via IPC."""
        if self.is_connected:
            result = await self._send_command_async(["set_property", "pause", False])
            if result:
                self._paused = False
                logger.info("Resumed via IPC")
            else:
                # Fallback - restart if IPC fails
                if self._current_url:
                    await self.play(self._current_url)
        self._paused = False

    async def toggle_pause(self) -> None:
        """Toggle pause state."""
        if self._paused:
            await self.resume()
        else:
            await self.pause()

    async def set_volume(self, volume: int) -> None:
        """Set volume."""
        self._volume = max(0, min(100, volume))
        if self.is_connected:
            await self._send_command_async(["set_property", "volume", self._volume])

    async def seek(self, seconds: float, relative: bool = True) -> None:
        """Seek in current track."""
        if self.is_connected:
            if relative:
                await self._send_command_async(["seek", seconds, "relative"])
            else:
                await self._send_command_async(["seek", seconds, "absolute"])

    async def get_position(self) -> float:
        """Get current playback position."""
        await self._update_properties()
        return self._position

    async def get_duration(self) -> float:
        """Get current track duration."""
        await self._update_properties()
        return self._duration

    async def shutdown(self) -> None:
        """Shutdown player."""
        await self.stop()

    def on_property_change(self, name: str, callback) -> None:
        """Register callback - not implemented yet."""
        pass

    async def get_property(self, name: str):
        """Get property."""
        if name == "volume":
            return self._volume
        if name == "pause":
            return self._paused
        if name == "time-pos":
            await self._update_properties()
            return self._position
        if name == "duration":
            await self._update_properties()
            return self._duration
        return None



"""Player screen for wrkmon."""

from textual.app import ComposeResult
from textual.screen import Screen
from textual.containers import Vertical, Horizontal
from textual.widgets import Static, ProgressBar, ListView, ListItem, Label
from textual.binding import Binding
from textual.reactive import reactive

from wrkmon.core.queue import QueueItem


class QueueListItem(ListItem):
    """A queue list item."""

    def __init__(self, item: QueueItem, index: int, is_current: bool = False, **kwargs):
        super().__init__(**kwargs)
        self.queue_item = item
        self.index = index
        self.is_current = is_current

    def compose(self) -> ComposeResult:
        from wrkmon.utils.stealth import get_stealth
        stealth = get_stealth()

        process_name = stealth.get_fake_process_name(self.queue_item.title)[:40]
        duration = stealth.format_duration(self.queue_item.duration)
        status = "RUNNING" if self.is_current else "QUEUED"

        prefix = ">" if self.is_current else " "
        text = f"{prefix} {self.index:2}  {process_name:<42} {duration:>8}  {status}"
        yield Label(text)


class PlayerScreen(Screen):
    """Player/Now Playing screen showing current track and queue."""

    BINDINGS = [
        Binding("escape", "go_back", "Back", show=False),
        Binding("space", "toggle_pause", "Play/Pause", show=False),
        Binding("+", "volume_up", "Volume Up", show=False),
        Binding("-", "volume_down", "Volume Down", show=False),
        Binding("n", "next_track", "Next", show=False),
        Binding("p", "prev_track", "Previous", show=False),
        Binding("s", "toggle_shuffle", "Shuffle", show=False),
        Binding("r", "toggle_repeat", "Repeat", show=False),
        Binding("c", "clear_queue", "Clear Queue", show=False),
    ]

    title = reactive("No process running")
    status = reactive("STOPPED")
    position = reactive(0.0)
    duration = reactive(0.0)
    volume = reactive(80)

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    def compose(self) -> ComposeResult:
        from wrkmon.utils.stealth import get_stealth
        self._stealth = get_stealth()

        with Vertical(id="player-screen"):
            # Now playing section
            yield Static("NOW RUNNING", id="now-playing-header")
            yield Static("", id="now-playing-title")

            # Progress
            with Horizontal(id="progress-section"):
                yield ProgressBar(total=100, show_percentage=False, id="progress-bar")
                yield Static("--:-- / --:--", id="progress-time")

            # Controls status
            with Horizontal(id="controls-section"):
                yield Static("VOL:", id="vol-label")
                yield ProgressBar(total=100, show_percentage=False, id="volume-bar")
                yield Static("80%", id="volume-value")
                yield Static("", id="shuffle-status")
                yield Static("", id="repeat-status")

            # Queue section
            yield Static("PROCESS QUEUE", id="queue-header")
            yield Static(
                " #   Process Name                                  Duration    Status",
                id="queue-list-header",
            )
            yield ListView(id="queue-list")

            # Status
            yield Static("", id="player-status")

    def on_mount(self) -> None:
        """Called when screen is mounted."""
        self.update_display()

    def update_display(self) -> None:
        """Update the entire display."""
        self.update_now_playing()
        self.update_queue()
        self.update_controls()

    def update_now_playing(self) -> None:
        """Update now playing display."""
        player = self.app.player
        queue = self.app.queue

        current = queue.current
        if current:
            process_name = self._stealth.get_fake_process_name(current.title)
            title_widget = self.query_one("#now-playing-title", Static)
            title_widget.update(f"  {process_name}")

            # Update progress
            pos = player.current_position
            dur = player.duration or current.duration
            self.update_progress(pos, dur)
        else:
            self.query_one("#now-playing-title", Static).update("  No process running")

    def update_progress(self, position: float, duration: float) -> None:
        """Update progress bar and time."""
        self.position = position
        self.duration = duration

        progress_bar = self.query_one("#progress-bar", ProgressBar)
        if duration > 0:
            progress_bar.update(progress=(position / duration) * 100)
        else:
            progress_bar.update(progress=0)

        pos_str = self._stealth.format_duration(position)
        dur_str = self._stealth.format_duration(duration)
        self.query_one("#progress-time", Static).update(f"{pos_str} / {dur_str}")

    def update_volume(self, volume: int) -> None:
        """Update volume display."""
        self.volume = volume
        self.query_one("#volume-bar", ProgressBar).update(progress=volume)
        self.query_one("#volume-value", Static).update(f"{volume}%")

    def update_controls(self) -> None:
        """Update control status indicators."""
        queue = self.app.queue

        shuffle_text = "[SHUF]" if queue.shuffle_mode else ""
        self.query_one("#shuffle-status", Static).update(shuffle_text)

        repeat_text = ""
        if queue.repeat_mode == "one":
            repeat_text = "[REP1]"
        elif queue.repeat_mode == "all":
            repeat_text = "[REPA]"
        self.query_one("#repeat-status", Static).update(repeat_text)

    def update_queue(self) -> None:
        """Update queue list."""
        queue = self.app.queue
        list_view = self.query_one("#queue-list", ListView)
        list_view.clear()

        items = queue.to_list()
        for i, item in enumerate(items):
            is_current = i == queue.current_index
            list_view.append(QueueListItem(item, i + 1, is_current))

        if not items:
            self.query_one("#player-status", Static).update("Queue empty")

    def update_status(self, message: str) -> None:
        """Update status message."""
        self.query_one("#player-status", Static).update(message)

    def action_go_back(self) -> None:
        """Go back to search screen."""
        self.app.pop_screen()

    async def action_toggle_pause(self) -> None:
        """Toggle play/pause."""
        await self.app.toggle_pause()
        status = "RUNNING" if self.app.player.is_playing else "SUSPENDED"
        self.update_status(f"Status: {status}")

    async def action_volume_up(self) -> None:
        """Increase volume."""
        new_vol = min(100, self.volume + 5)
        await self.app.set_volume(new_vol)
        self.update_volume(new_vol)

    async def action_volume_down(self) -> None:
        """Decrease volume."""
        new_vol = max(0, self.volume - 5)
        await self.app.set_volume(new_vol)
        self.update_volume(new_vol)

    async def action_next_track(self) -> None:
        """Play next track."""
        await self.app.play_next()
        self.update_display()

    async def action_prev_track(self) -> None:
        """Play previous track."""
        await self.app.play_previous()
        self.update_display()

    def action_toggle_shuffle(self) -> None:
        """Toggle shuffle mode."""
        is_shuffle = self.app.queue.toggle_shuffle()
        self.update_controls()
        self.update_status(f"Shuffle: {'ON' if is_shuffle else 'OFF'}")

    def action_toggle_repeat(self) -> None:
        """Toggle repeat mode."""
        mode = self.app.queue.cycle_repeat()
        self.update_controls()
        mode_names = {"none": "OFF", "one": "ONE", "all": "ALL"}
        self.update_status(f"Repeat: {mode_names[mode]}")

    def action_clear_queue(self) -> None:
        """Clear the queue."""
        self.app.queue.clear()
        self.update_queue()
        self.update_status("Queue cleared")



"""Tests for core modules."""

import pytest
from wrkmon.core.queue import PlayQueue, QueueItem


class TestPlayQueue:
    """Tests for PlayQueue."""

    def test_empty_queue(self):
        queue = PlayQueue()
        assert queue.is_empty
        assert queue.length == 0
        assert queue.current is None

    def test_add_item(self):
        queue = PlayQueue()
        item = QueueItem(
            video_id="test123",
            title="Test Track",
            channel="Test Channel",
            duration=180,
        )
        pos = queue.add(item)
        assert pos == 0
        assert queue.length == 1
        assert not queue.is_empty

    def test_next_previous(self):
        queue = PlayQueue()
        for i in range(3):
            queue.add(QueueItem(
                video_id=f"test{i}",
                title=f"Track {i}",
                channel="Channel",
                duration=100,
            ))

        # Start at beginning
        queue.jump_to(0)
        assert queue.current.video_id == "test0"

        # Next
        queue.next()
        assert queue.current.video_id == "test1"

        # Previous
        queue.previous()
        assert queue.current.video_id == "test0"

    def test_shuffle(self):
        queue = PlayQueue()
        for i in range(5):
            queue.add(QueueItem(
                video_id=f"test{i}",
                title=f"Track {i}",
                channel="Channel",
                duration=100,
            ))

        queue.shuffle()
        assert queue.shuffle_mode
        assert len(queue._shuffle_order) == 5

        queue.unshuffle()
        assert not queue.shuffle_mode

    def test_repeat_modes(self):
        queue = PlayQueue()
        assert queue.repeat_mode == "none"

        queue.cycle_repeat()
        assert queue.repeat_mode == "one"

        queue.cycle_repeat()
        assert queue.repeat_mode == "all"

        queue.cycle_repeat()
        assert queue.repeat_mode == "none"

    def test_clear(self):
        queue = PlayQueue()
        queue.add(QueueItem(
            video_id="test",
            title="Track",
            channel="Channel",
            duration=100,
        ))
        assert queue.length == 1

        queue.clear()
        assert queue.is_empty


class TestQueueItem:
    """Tests for QueueItem."""

    def test_url_property(self):
        item = QueueItem(
            video_id="abc123",
            title="Test",
            channel="Channel",
            duration=60,
        )
        assert item.url == "https://www.youtube.com/watch?v=abc123"



"""Core functionality for wrkmon."""

from wrkmon.core.youtube import YouTubeClient
from wrkmon.core.player import AudioPlayer
from wrkmon.core.queue import PlayQueue
from wrkmon.core.cache import Cache

__all__ = ["YouTubeClient", "AudioPlayer", "PlayQueue", "Cache"]



"""ASCII thumbnail widget for wrkmon - with color support."""

import asyncio
from textual.app import ComposeResult
from textual.containers import Container
from textual.reactive import reactive
from textual.widgets import Static

from wrkmon.utils.ascii_art import get_or_fetch_ascii, get_cached_ascii, clear_cache


class ThumbnailPreview(Static):
    """Widget to display colored ASCII art thumbnail preview."""

    DEFAULT_CSS = """
    ThumbnailPreview {
        width: 100%;
        height: auto;
        min-height: 8;
        max-height: 18;
        background: #0d1117;
        padding: 0;
    }

    ThumbnailPreview.loading {
        color: #6e7681;
    }

    ThumbnailPreview.hidden {
        display: none;
    }
    """

    video_id = reactive("")
    is_loading = reactive(False)
    style_mode = reactive("colored_blocks")  # colored_blocks, colored_simple, blocks, braille

    def __init__(
        self,
        video_id: str = "",
        width: int = 45,
        style: str = "colored_blocks",
        **kwargs
    ) -> None:
        super().__init__(**kwargs)
        self._ascii_width = width
        self._current_task: asyncio.Task | None = None
        self.style_mode = style
        if video_id:
            self.video_id = video_id

    def compose(self) -> ComposeResult:
        yield Static("", id="ascii-content", markup=True)

    def on_mount(self) -> None:
        """Load thumbnail on mount if video_id is set."""
        if self.video_id:
            self._load_thumbnail()

    def watch_video_id(self, video_id: str) -> None:
        """React to video_id changes."""
        if video_id:
            self._load_thumbnail()
        else:
            self._clear_thumbnail()

    def _load_thumbnail(self) -> None:
        """Load ASCII thumbnail for current video."""
        if not self.video_id:
            return

        # Check cache first
        cache_key = f"{self.video_id}_{self.style_mode}_{self._ascii_width}"
        from wrkmon.utils.ascii_art import _thumbnail_cache
        cached = _thumbnail_cache.get(cache_key)
        if cached:
            self._display_ascii(cached)
            return

        # Load async
        self.is_loading = True
        self.add_class("loading")
        self._update_content("[dim]Loading...[/]")

        # Cancel any existing task
        if self._current_task and not self._current_task.done():
            self._current_task.cancel()

        # Start new load task
        self._current_task = asyncio.create_task(self._fetch_and_display())

    async def _fetch_and_display(self) -> None:
        """Fetch thumbnail and display it."""
        try:
            ascii_art = await get_or_fetch_ascii(
                self.video_id,
                width=self._ascii_width,
                style=self.style_mode,
            )

            if ascii_art:
                self._display_ascii(ascii_art)
            else:
                self._update_content("[dim]No thumbnail[/]")

        except asyncio.CancelledError:
            pass
        except Exception as e:
            self._update_content(f"[red]Error: {e}[/]")
        finally:
            self.is_loading = False
            self.remove_class("loading")

    def _display_ascii(self, ascii_art: str) -> None:
        """Display ASCII art in the widget."""
        self._update_content(ascii_art)

    def _update_content(self, content: str) -> None:
        """Update the content display."""
        try:
            widget = self.query_one("#ascii-content", Static)
            widget.update(content)
        except Exception:
            pass

    def _clear_thumbnail(self) -> None:
        """Clear the thumbnail display."""
        self._update_content("")

    def set_video(self, video_id: str) -> None:
        """Set video ID and load thumbnail."""
        self.video_id = video_id

    def set_style(self, style: str) -> None:
        """Change the rendering style and reload."""
        if style != self.style_mode:
            self.style_mode = style
            if self.video_id:
                self._load_thumbnail()

    def clear(self) -> None:
        """Clear the thumbnail."""
        self.video_id = ""

    def show(self) -> None:
        """Show the widget."""
        self.remove_class("hidden")

    def hide(self) -> None:
        """Hide the widget."""
        self.add_class("hidden")

    def cycle_style(self) -> str:
        """Cycle through rendering styles."""
        styles = ["colored_blocks", "colored_simple", "braille", "blocks"]
        current_idx = styles.index(self.style_mode) if self.style_mode in styles else 0
        next_idx = (current_idx + 1) % len(styles)
        self.set_style(styles[next_idx])
        return styles[next_idx]


class ThumbnailPanel(Container):
    """Panel containing thumbnail preview with title."""

    DEFAULT_CSS = """
    ThumbnailPanel {
        width: 100%;
        height: auto;
        background: #161b22;
        border: solid #30363d;
        padding: 1;
    }

    ThumbnailPanel > #panel-title {
        height: 1;
        color: #58a6ff;
        text-style: bold;
        margin-bottom: 1;
    }

    ThumbnailPanel.hidden {
        display: none;
    }
    """

    def __init__(self, title: str = "Preview", width: int = 45, **kwargs) -> None:
        super().__init__(**kwargs)
        self._title = title
        self._width = width

    def compose(self) -> ComposeResult:
        yield Static(self._title, id="panel-title", markup=True)
        yield ThumbnailPreview(width=self._width, id="thumbnail-preview")

    def set_video(self, video_id: str, title: str = "") -> None:
        """Set video to preview."""
        if title:
            try:
                display_title = title[:40] + "..." if len(title) > 40 else title
                self.query_one("#panel-title", Static).update(f"[bold]{display_title}[/]")
            except Exception:
                pass

        try:
            self.query_one("#thumbnail-preview", ThumbnailPreview).set_video(video_id)
        except Exception:
            pass

    def clear(self) -> None:
        """Clear the preview."""
        try:
            self.query_one("#thumbnail-preview", ThumbnailPreview).clear()
            self.query_one("#panel-title", Static).update("Preview")
        except Exception:
            pass

    def cycle_style(self) -> str:
        """Cycle the thumbnail style."""
        try:
            return self.query_one("#thumbnail-preview", ThumbnailPreview).cycle_style()
        except Exception:
            return "colored_blocks"

    def show(self) -> None:
        """Show the panel."""
        self.remove_class("hidden")

    def hide(self) -> None:
        """Hide the panel."""
        self.add_class("hidden")



"""History view container for wrkmon."""

from textual.app import ComposeResult
from textual.containers import Vertical
from textual.widgets import Static, ListView
from textual.binding import Binding
from textual import on

from wrkmon.core.youtube import SearchResult
from wrkmon.data.models import HistoryEntry
from wrkmon.ui.widgets.result_item import HistoryItem
from wrkmon.ui.messages import TrackSelected, TrackQueued, StatusMessage


class HistoryView(Vertical):
    """History view - shows recently played tracks."""

    BINDINGS = [
        Binding("a", "queue_selected", "Add to Queue", show=True),
        Binding("c", "clear_history", "Clear All", show=True),
        Binding("r", "refresh", "Refresh", show=True),
    ]

    def __init__(self, **kwargs) -> None:
        super().__init__(**kwargs)
        self.entries: list[HistoryEntry] = []

    def compose(self) -> ComposeResult:
        yield Static("HISTORY", id="view-title")

        yield Static(
            " #   Process                              Duration  Runs  Last Run",
            id="list-header",
        )

        yield ListView(id="history-list")
        yield Static("Loading history...", id="status-bar")

    def on_mount(self) -> None:
        """Load history on mount."""
        self.load_history()

    def load_history(self) -> None:
        """Load play history from database."""
        try:
            db = self.app.database
            self.entries = db.get_history(limit=100)
            self._display_history()
        except Exception as e:
            self._update_status(f"Failed to load history: {e}")

    def _display_history(self) -> None:
        """Display history entries."""
        list_view = self.query_one("#history-list", ListView)
        list_view.clear()

        if not self.entries:
            self._update_status("No history yet")
            return

        for i, entry in enumerate(self.entries, 1):
            track = entry.track
            last_played = entry.played_at.strftime("%m-%d %H:%M")

            list_view.append(
                HistoryItem(
                    title=track.title,
                    duration=track.duration,
                    play_count=entry.play_count,
                    last_played=last_played,
                    index=i,
                    video_id=track.video_id,
                    channel=track.channel,
                )
            )

        total_plays = sum(e.play_count for e in self.entries)
        self._update_status(f"{len(self.entries)} tracks, {total_plays} total plays")

    def _update_status(self, message: str) -> None:
        """Update status bar."""
        try:
            self.query_one("#status-bar", Static).update(message)
        except Exception:
            pass

    def _get_selected(self) -> HistoryItem | None:
        """Get the currently selected history item."""
        list_view = self.query_one("#history-list", ListView)
        if list_view.highlighted_child is None:
            return None

        item = list_view.highlighted_child
        if isinstance(item, HistoryItem):
            return item
        return None

    def _item_to_result(self, item: HistoryItem) -> SearchResult:
        """Convert a history item to a SearchResult."""
        return SearchResult(
            video_id=item.video_id,
            title=item.title,
            channel=item.channel,
            duration=item.duration,
            view_count=0,
        )

    @on(ListView.Selected, "#history-list")
    def handle_item_selected(self, event: ListView.Selected) -> None:
        """Handle Enter key on a history item - play it."""
        if isinstance(event.item, HistoryItem):
            result = self._item_to_result(event.item)
            self.post_message(TrackSelected(result))
            self._update_status(f"Playing: {event.item.title[:40]}...")

    def action_queue_selected(self) -> None:
        """Add selected track to queue."""
        item = self._get_selected()
        if item:
            result = self._item_to_result(item)
            self.post_message(TrackQueued(result))
            self._update_status(f"Queued: {item.title[:40]}...")

    def action_clear_history(self) -> None:
        """Clear all history."""
        try:
            db = self.app.database
            count = db.clear_history()
            self.entries = []
            self._display_history()
            self.post_message(StatusMessage(f"Cleared {count} entries", "info"))
        except Exception as e:
            self.post_message(StatusMessage(f"Error: {e}", "error"))

    def action_refresh(self) -> None:
        """Refresh history."""
        self.load_history()
        self.post_message(StatusMessage("History refreshed", "info"))



"""Help overlay screen for wrkmon."""

from textual.app import ComposeResult
from textual.containers import Container, Vertical, VerticalScroll
from textual.screen import ModalScreen
from textual.widgets import Static
from textual.binding import Binding


HELP_TEXT = """
[bold cyan]━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━[/]
[bold cyan]                    WRKMON KEYBOARD SHORTCUTS[/]
[bold cyan]━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━[/]

[bold orange1]NAVIGATION[/]
  [bold purple]F1[/]          Switch to Search view
  [bold purple]F2[/]          Switch to Queue view
  [bold purple]F3[/]          Switch to History view
  [bold purple]F4[/]          Switch to Playlists view
  [bold purple]j / ↓[/]       Move down in list
  [bold purple]k / ↑[/]       Move up in list
  [bold purple]g / Home[/]    Jump to top of list
  [bold purple]G / End[/]     Jump to bottom of list
  [bold purple]Tab[/]         Next focusable element
  [bold purple]Shift+Tab[/]   Previous focusable element

[bold orange1]PLAYBACK[/]
  [bold purple]F5 / Space[/]  Play / Pause
  [bold purple]F6 / -[/]      Volume down
  [bold purple]F7 / + =[/]    Volume up
  [bold purple]F8 / n[/]      Next track
  [bold purple]p[/]           Previous track
  [bold purple]F9 / s[/]      Stop playback
  [bold purple]r[/]           Cycle repeat mode (Off → One → All)
  [bold purple]m[/]           Mute / Unmute

[bold orange1]SEARCH & QUEUE[/]
  [bold purple]/[/]           Focus search input
  [bold purple]Enter[/]       Play selected track
  [bold purple]a[/]           Add selected to queue
  [bold purple]F10[/]         Add highlighted to queue
  [bold purple]d / Del[/]     Remove from queue
  [bold purple]c[/]           Clear queue

[bold orange1]VIEW OPTIONS[/]
  [bold purple]t[/]           Toggle thumbnail preview
  [bold purple]?[/]           Show this help
  [bold purple]Escape[/]      Close overlay / Go back

[bold orange1]APPLICATION[/]
  [bold purple]Ctrl+Q[/]      Quit application
  [bold purple]Ctrl+C[/]      Quit application

[bold cyan]━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━[/]
[dim]Press [bold]Escape[/bold] or [bold]?[/bold] to close this help[/]
"""


class HelpScreen(ModalScreen):
    """Modal help screen showing keyboard shortcuts."""

    BINDINGS = [
        Binding("escape", "close", "Close", show=False),
        Binding("?", "close", "Close", show=False),
        Binding("q", "close", "Close", show=False),
    ]

    def compose(self) -> ComposeResult:
        with Container(id="help-container"):
            yield Static("[bold cyan]⌨  KEYBOARD SHORTCUTS[/]", id="help-title")
            with VerticalScroll(id="help-scroll"):
                yield Static(HELP_TEXT, id="help-content", markup=True)

    def action_close(self) -> None:
        """Close the help screen."""
        self.dismiss()

    def on_click(self) -> None:
        """Close on click outside."""
        self.dismiss()



"""Reusable TUI components for wrkmon."""

from textual.app import ComposeResult
from textual.containers import Horizontal, Vertical
from textual.widgets import Static, Input, ProgressBar, Label
from textual.reactive import reactive

from wrkmon.utils.stealth import get_stealth


class HeaderBar(Static):
    """Header bar showing app name and fake system stats."""

    cpu = reactive(23)
    mem = reactive(45)

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self._stealth = get_stealth()

    def compose(self) -> ComposeResult:
        with Horizontal(id="header"):
            yield Static("WRKMON v1.0.0", id="header-title")
            yield Static(self._get_stats_text(), id="header-stats")

    def _get_stats_text(self) -> str:
        return f"[CPU: {self.cpu}%] [MEM: {self.mem}%]"

    def update_stats(self) -> None:
        """Update fake system stats."""
        self.cpu = self._stealth.get_fake_cpu()
        self.mem = self._stealth.get_fake_memory()
        stats = self.query_one("#header-stats", Static)
        stats.update(self._get_stats_text())


class SearchBar(Static):
    """Search input bar."""

    def compose(self) -> ComposeResult:
        with Horizontal(id="search-container"):
            yield Static("> Search: ", classes="search-label")
            yield Input(placeholder="Enter search query...", id="search-input")

    def focus_input(self) -> None:
        """Focus the search input."""
        self.query_one("#search-input", Input).focus()

    def clear_input(self) -> None:
        """Clear the search input."""
        self.query_one("#search-input", Input).value = ""

    @property
    def value(self) -> str:
        """Get current search value."""
        return self.query_one("#search-input", Input).value


class ResultsHeader(Static):
    """Header row for results list (styled like ps/top output)."""

    def compose(self) -> ComposeResult:
        with Horizontal(id="results-header"):
            yield Static("#", classes="result-index")
            yield Static("Process Name", classes="result-title")
            yield Static("PID", classes="result-pid")
            yield Static("Status", classes="result-status")


class ResultItem(Static):
    """A single result item styled like a process entry."""

    def __init__(
        self,
        index: int,
        title: str,
        video_id: str,
        status: str = "READY",
        **kwargs,
    ):
        super().__init__(**kwargs)
        self.index = index
        self.title = title
        self.video_id = video_id
        self.status = status
        self._stealth = get_stealth()

    def compose(self) -> ComposeResult:
        fake_pid = self._stealth.get_fake_pid()
        process_name = self._stealth.get_fake_process_name(self.title)

        with Horizontal(classes="result-item"):
            yield Static(f"{self.index}", classes="result-index")
            yield Static(process_name[:40], classes="result-title")
            yield Static(str(fake_pid), classes="result-pid")
            status_class = f"status-{self.status.lower()}"
            yield Static(self.status, classes=f"result-status {status_class}")


class PlayerBar(Static):
    """Bottom player bar showing current track and controls."""

    title = reactive("No process running")
    status = reactive("STOPPED")
    position = reactive(0.0)
    duration = reactive(0.0)
    volume = reactive(80)

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self._stealth = get_stealth()

    def compose(self) -> ComposeResult:
        with Vertical(id="player-bar"):
            # Now playing line
            with Horizontal(id="now-playing"):
                yield Static("NOW:", id="now-playing-label")
                yield Static(self.title, id="now-playing-title")
                yield Static(self._get_status_icon(), id="now-playing-status")

            # Progress bar
            with Horizontal(id="progress-container"):
                yield ProgressBar(total=100, show_percentage=False, id="progress-bar")
                yield Static(self._get_time_text(), id="progress-time")

            # Volume bar
            with Horizontal(id="volume-container"):
                yield Static("VOL:", id="volume-label")
                yield ProgressBar(total=100, show_percentage=False, id="volume-bar")
                yield Static(f"{self.volume}%", id="volume-value")

    def _get_status_icon(self) -> str:
        icons = {
            "playing": "▶",
            "paused": "⏸",
            "stopped": "⏹",
            "buffering": "⟳",
        }
        return icons.get(self.status.lower(), "⏹")

    def _get_time_text(self) -> str:
        pos = self._stealth.format_duration(self.position)
        dur = self._stealth.format_duration(self.duration)
        return f"{pos} / {dur}"

    def update_now_playing(self, title: str, status: str) -> None:
        """Update the now playing display."""
        self.title = title
        self.status = status
        process_name = self._stealth.get_fake_process_name(title)
        self.query_one("#now-playing-title", Static).update(f"{process_name}")
        self.query_one("#now-playing-status", Static).update(self._get_status_icon())

    def update_progress(self, position: float, duration: float) -> None:
        """Update the progress bar."""
        self.position = position
        self.duration = duration

        progress_bar = self.query_one("#progress-bar", ProgressBar)
        if duration > 0:
            progress_bar.update(progress=(position / duration) * 100)
        else:
            progress_bar.update(progress=0)

        self.query_one("#progress-time", Static).update(self._get_time_text())

    def update_volume(self, volume: int) -> None:
        """Update the volume display."""
        self.volume = volume
        self.query_one("#volume-bar", ProgressBar).update(progress=volume)
        self.query_one("#volume-value", Static).update(f"{volume}%")


class FooterBar(Static):
    """Footer showing keyboard shortcuts."""

    def __init__(self, hints: list[tuple[str, str]] = None, **kwargs):
        super().__init__(**kwargs)
        self.hints = hints or [
            ("/", "Search"),
            ("Space", "Play/Pause"),
            ("+/-", "Volume"),
            ("n/p", "Next/Prev"),
            ("q", "Queue"),
            ("h", "History"),
            ("Esc", "Back"),
            ("Ctrl+C", "Quit"),
        ]

    def compose(self) -> ComposeResult:
        with Horizontal(id="footer"):
            for key, action in self.hints:
                yield Static(f"[{key}] {action}", classes="key-hint")


class LoadingIndicator(Static):
    """Loading indicator."""

    def compose(self) -> ComposeResult:
        yield Static("Loading...", classes="loading")


class EmptyState(Static):
    """Empty state message."""

    def __init__(self, message: str = "No data", **kwargs):
        super().__init__(**kwargs)
        self.message = message

    def compose(self) -> ComposeResult:
        yield Static(self.message, classes="empty-state")



"""ASCII art generator for YouTube thumbnails - Enhanced with colors and blocks."""

import asyncio
import io
import logging
import urllib.request
from pathlib import Path
from typing import Optional

logger = logging.getLogger("wrkmon.ascii_art")

# Character sets for different styles
CHARS_BLOCKS = "█▓▒░ "  # Block characters (best for images)
CHARS_BLOCKS_DETAILED = "████▓▓▒▒░░  "  # More gradients
CHARS_ASCII = "@%#*+=-:. "  # Classic ASCII
CHARS_BRAILLE = "⣿⣷⣯⣟⡿⢿⣻⣽⣾⣶⣦⣤⣀⡀ "  # Braille patterns

# Color palette for Rich markup (approximate colors)
COLOR_PALETTE = [
    "#000000", "#1a1a1a", "#333333", "#4d4d4d", "#666666",
    "#808080", "#999999", "#b3b3b3", "#cccccc", "#e6e6e6", "#ffffff"
]

# Extended color palette for better color representation
EXTENDED_COLORS = {
    (0, 0, 0): "black",
    (255, 255, 255): "white",
    (255, 0, 0): "red",
    (0, 255, 0): "green",
    (0, 0, 255): "blue",
    (255, 255, 0): "yellow",
    (255, 0, 255): "magenta",
    (0, 255, 255): "cyan",
    (128, 128, 128): "grey50",
    (192, 192, 192): "grey74",
    (128, 0, 0): "dark_red",
    (0, 128, 0): "dark_green",
    (0, 0, 128): "dark_blue",
    (128, 128, 0): "olive",
    (128, 0, 128): "purple",
    (0, 128, 128): "teal",
}


def get_thumbnail_url(video_id: str, quality: str = "mqdefault") -> str:
    """
    Get YouTube thumbnail URL for a video.

    Quality options:
    - default: 120x90
    - mqdefault: 320x180
    - hqdefault: 480x360
    - sddefault: 640x480
    - maxresdefault: 1280x720
    """
    return f"https://img.youtube.com/vi/{video_id}/{quality}.jpg"


def download_thumbnail(video_id: str, quality: str = "mqdefault") -> Optional[bytes]:
    """Download thumbnail image from YouTube."""
    url = get_thumbnail_url(video_id, quality)
    try:
        req = urllib.request.Request(
            url,
            headers={"User-Agent": "Mozilla/5.0"}
        )
        with urllib.request.urlopen(req, timeout=5) as response:
            return response.read()
    except Exception as e:
        logger.debug(f"Failed to download thumbnail: {e}")
        return None


async def download_thumbnail_async(video_id: str, quality: str = "mqdefault") -> Optional[bytes]:
    """Async version of download_thumbnail."""
    return await asyncio.to_thread(download_thumbnail, video_id, quality)


def find_closest_color(r: int, g: int, b: int) -> str:
    """Find the closest named color for Rich markup."""
    min_dist = float('inf')
    closest = "white"

    for (cr, cg, cb), name in EXTENDED_COLORS.items():
        dist = (r - cr) ** 2 + (g - cg) ** 2 + (b - cb) ** 2
        if dist < min_dist:
            min_dist = dist
            closest = name

    return closest


def rgb_to_hex(r: int, g: int, b: int) -> str:
    """Convert RGB to hex color."""
    return f"#{r:02x}{g:02x}{b:02x}"


def image_to_ascii_blocks(
    image_data: bytes,
    width: int = 40,
    colored: bool = True,
) -> str:
    """
    Convert image to colored block ASCII art.

    Uses half-block characters (▀▄) to achieve 2x vertical resolution
    with foreground and background colors.
    """
    try:
        from PIL import Image
    except ImportError:
        logger.warning("Pillow not installed")
        return ""

    try:
        img = Image.open(io.BytesIO(image_data))

        # Calculate dimensions (2 pixels per character vertically)
        aspect_ratio = img.height / img.width
        char_height = int(width * aspect_ratio * 0.5)  # Half because we use half-blocks
        pixel_height = char_height * 2

        # Resize
        img = img.resize((width, pixel_height))
        img = img.convert("RGB")
        pixels = list(img.getdata())

        lines = []
        for y in range(0, pixel_height, 2):
            line = ""
            for x in range(width):
                # Top pixel
                top_idx = y * width + x
                # Bottom pixel
                bot_idx = (y + 1) * width + x if (y + 1) < pixel_height else top_idx

                top_r, top_g, top_b = pixels[top_idx]
                bot_r, bot_g, bot_b = pixels[bot_idx]

                if colored:
                    # Use half-block with fg (top) and bg (bottom) colors
                    top_hex = rgb_to_hex(top_r, top_g, top_b)
                    bot_hex = rgb_to_hex(bot_r, bot_g, bot_b)
                    line += f"[{top_hex} on {bot_hex}]▀[/]"
                else:
                    # Grayscale block
                    avg = (top_r + top_g + top_b + bot_r + bot_g + bot_b) // 6
                    char_idx = int(avg / 256 * len(CHARS_BLOCKS))
                    char_idx = min(char_idx, len(CHARS_BLOCKS) - 1)
                    line += CHARS_BLOCKS[char_idx]

            lines.append(line)

        return "\n".join(lines)

    except Exception as e:
        logger.debug(f"Failed to convert image: {e}")
        return ""


def image_to_ascii_simple(
    image_data: bytes,
    width: int = 40,
    chars: str = CHARS_BLOCKS,
) -> str:
    """
    Simple grayscale ASCII art conversion.
    """
    try:
        from PIL import Image
    except ImportError:
        return ""

    try:
        img = Image.open(io.BytesIO(image_data))

        aspect_ratio = img.height / img.width
        height = int(width * aspect_ratio * 0.5)

        img = img.resize((width, height))
        img = img.convert("L")  # Grayscale

        pixels = list(img.getdata())

        lines = []
        for i in range(0, len(pixels), width):
            line = ""
            for pixel in pixels[i:i + width]:
                char_idx = int((255 - pixel) / 256 * len(chars))
                char_idx = min(char_idx, len(chars) - 1)
                line += chars[char_idx]
            lines.append(line)

        return "\n".join(lines)

    except Exception as e:
        logger.debug(f"Failed to convert image: {e}")
        return ""


def image_to_ascii_colored_simple(
    image_data: bytes,
    width: int = 40,
) -> str:
    """
    Colored ASCII using full blocks with single color per character.
    Simpler than half-blocks but still colorful.
    """
    try:
        from PIL import Image
    except ImportError:
        return ""

    try:
        img = Image.open(io.BytesIO(image_data))

        aspect_ratio = img.height / img.width
        height = int(width * aspect_ratio * 0.5)

        img = img.resize((width, height))
        img = img.convert("RGB")

        pixels = list(img.getdata())

        lines = []
        for i in range(0, len(pixels), width):
            line = ""
            for r, g, b in pixels[i:i + width]:
                hex_color = rgb_to_hex(r, g, b)
                line += f"[{hex_color}]█[/]"
            lines.append(line)

        return "\n".join(lines)

    except Exception as e:
        logger.debug(f"Failed to convert image: {e}")
        return ""


def image_to_braille(
    image_data: bytes,
    width: int = 60,
    threshold: int = 128,
) -> str:
    """
    Convert image to braille pattern art.
    Each braille character represents a 2x4 pixel grid.
    """
    try:
        from PIL import Image
    except ImportError:
        return ""

    try:
        img = Image.open(io.BytesIO(image_data))

        # Braille is 2 dots wide, 4 dots tall per character
        char_width = width
        aspect_ratio = img.height / img.width
        char_height = int(char_width * aspect_ratio * 0.5)

        pixel_width = char_width * 2
        pixel_height = char_height * 4

        img = img.resize((pixel_width, pixel_height))
        img = img.convert("L")  # Grayscale

        pixels = list(img.getdata())

        def get_pixel(x, y):
            if 0 <= x < pixel_width and 0 <= y < pixel_height:
                return pixels[y * pixel_width + x] < threshold
            return False

        # Braille dot positions
        # 1 4
        # 2 5
        # 3 6
        # 7 8
        dot_map = [
            (0, 0, 0x01), (1, 0, 0x08),
            (0, 1, 0x02), (1, 1, 0x10),
            (0, 2, 0x04), (1, 2, 0x20),
            (0, 3, 0x40), (1, 3, 0x80),
        ]

        lines = []
        for cy in range(char_height):
            line = ""
            for cx in range(char_width):
                px = cx * 2
                py = cy * 4

                code = 0x2800  # Braille base
                for dx, dy, bit in dot_map:
                    if get_pixel(px + dx, py + dy):
                        code |= bit

                line += chr(code)
            lines.append(line)

        return "\n".join(lines)

    except Exception as e:
        logger.debug(f"Failed to convert image: {e}")
        return ""


def video_thumbnail_to_ascii(
    video_id: str,
    width: int = 40,
    quality: str = "mqdefault",
    style: str = "colored_blocks",  # colored_blocks, colored_simple, blocks, braille
) -> str:
    """
    Download YouTube thumbnail and convert to ASCII art.

    Styles:
    - colored_blocks: Half-block characters with full color (best quality)
    - colored_simple: Full blocks with color
    - blocks: Grayscale block characters
    - braille: Braille dot patterns (high detail, monochrome)
    """
    image_data = download_thumbnail(video_id, quality)
    if not image_data:
        return ""

    if style == "colored_blocks":
        return image_to_ascii_blocks(image_data, width=width, colored=True)
    elif style == "colored_simple":
        return image_to_ascii_colored_simple(image_data, width=width)
    elif style == "braille":
        return image_to_braille(image_data, width=width)
    else:  # blocks
        return image_to_ascii_simple(image_data, width=width)


async def video_thumbnail_to_ascii_async(
    video_id: str,
    width: int = 40,
    quality: str = "mqdefault",
    style: str = "colored_blocks",
) -> str:
    """Async version of video_thumbnail_to_ascii."""
    image_data = await download_thumbnail_async(video_id, quality)
    if not image_data:
        return ""

    if style == "colored_blocks":
        return await asyncio.to_thread(image_to_ascii_blocks, image_data, width, True)
    elif style == "colored_simple":
        return await asyncio.to_thread(image_to_ascii_colored_simple, image_data, width)
    elif style == "braille":
        return await asyncio.to_thread(image_to_braille, image_data, width)
    else:
        return await asyncio.to_thread(image_to_ascii_simple, image_data, width)


# Cache for thumbnails
_thumbnail_cache: dict[str, str] = {}
_cache_max_size = 50


def get_cached_ascii(video_id: str) -> Optional[str]:
    """Get cached ASCII art for a video."""
    return _thumbnail_cache.get(video_id)


def cache_ascii(video_id: str, ascii_art: str) -> None:
    """Cache ASCII art for a video."""
    global _thumbnail_cache

    if len(_thumbnail_cache) >= _cache_max_size:
        oldest = next(iter(_thumbnail_cache))
        del _thumbnail_cache[oldest]

    _thumbnail_cache[video_id] = ascii_art


def clear_cache() -> None:
    """Clear the thumbnail cache."""
    global _thumbnail_cache
    _thumbnail_cache = {}


async def get_or_fetch_ascii(
    video_id: str,
    width: int = 40,
    quality: str = "mqdefault",
    style: str = "colored_blocks",
) -> str:
    """Get ASCII art from cache or fetch it."""
    cache_key = f"{video_id}_{style}_{width}"
    cached = _thumbnail_cache.get(cache_key)
    if cached:
        return cached

    ascii_art = await video_thumbnail_to_ascii_async(
        video_id,
        width=width,
        quality=quality,
        style=style,
    )

    if ascii_art:
        _thumbnail_cache[cache_key] = ascii_art

    return ascii_art



"""Cross-platform media key support for wrkmon.

Linux: Uses MPRIS D-Bus interface (standard way for desktop environments)
Windows/macOS: Uses pynput for global hotkey listening
"""

import asyncio
import logging
import sys
from typing import Callable, Optional, Any

logger = logging.getLogger("wrkmon.media_keys")

# Platform detection
IS_LINUX = sys.platform == "linux"
IS_WINDOWS = sys.platform == "win32"
IS_MACOS = sys.platform == "darwin"

# Check available backends
MPRIS_AVAILABLE = False
PYNPUT_AVAILABLE = False

if IS_LINUX:
    try:
        from dbus_next.aio import MessageBus
        from dbus_next.service import ServiceInterface, method, dbus_property
        from dbus_next import Variant, BusType
        MPRIS_AVAILABLE = True
    except ImportError:
        logger.debug("dbus-next not installed, MPRIS support disabled")

if IS_WINDOWS or IS_MACOS:
    try:
        from pynput import keyboard
        PYNPUT_AVAILABLE = True
    except ImportError:
        logger.debug("pynput not installed, global hotkey support disabled")


# ============================================
# MPRIS Implementation (Linux)
# ============================================

if MPRIS_AVAILABLE:
    class MPRISRootInterface(ServiceInterface):
        """MPRIS MediaPlayer2 root interface."""

        def __init__(self, callback: Callable):
            super().__init__("org.mpris.MediaPlayer2")
            self._callback = callback

        @method()
        def Raise(self):
            pass

        @method()
        def Quit(self):
            asyncio.create_task(self._callback("quit"))

        @dbus_property()
        def CanQuit(self) -> "b":
            return True

        @dbus_property()
        def CanRaise(self) -> "b":
            return False

        @dbus_property()
        def HasTrackList(self) -> "b":
            return False

        @dbus_property()
        def Identity(self) -> "s":
            return "wrkmon"

        @dbus_property()
        def DesktopEntry(self) -> "s":
            return "wrkmon"

        @dbus_property()
        def SupportedUriSchemes(self) -> "as":
            return ["https", "http"]

        @dbus_property()
        def SupportedMimeTypes(self) -> "as":
            return ["audio/mpeg", "audio/ogg", "audio/webm"]


    class MPRISPlayerInterface(ServiceInterface):
        """MPRIS MediaPlayer2.Player interface."""

        def __init__(self, callback: Callable):
            super().__init__("org.mpris.MediaPlayer2.Player")
            self._callback = callback
            self._is_playing = False
            self._position = 0
            self._metadata: dict = {}
            self._volume = 1.0

        @method()
        def Next(self):
            logger.info("MPRIS: Next track")
            asyncio.create_task(self._callback("next"))

        @method()
        def Previous(self):
            logger.info("MPRIS: Previous track")
            asyncio.create_task(self._callback("previous"))

        @method()
        def Pause(self):
            logger.info("MPRIS: Pause")
            asyncio.create_task(self._callback("pause"))

        @method()
        def PlayPause(self):
            logger.info("MPRIS: PlayPause")
            asyncio.create_task(self._callback("play_pause"))

        @method()
        def Stop(self):
            logger.info("MPRIS: Stop")
            asyncio.create_task(self._callback("stop"))

        @method()
        def Play(self):
            logger.info("MPRIS: Play")
            asyncio.create_task(self._callback("play"))

        @method()
        def Seek(self, offset: "x"):
            asyncio.create_task(self._callback("seek", offset / 1_000_000))

        @method()
        def SetPosition(self, track_id: "o", position: "x"):
            asyncio.create_task(self._callback("set_position", position / 1_000_000))

        @dbus_property()
        def PlaybackStatus(self) -> "s":
            return "Playing" if self._is_playing else "Paused"

        @dbus_property()
        def Rate(self) -> "d":
            return 1.0

        @dbus_property()
        def Metadata(self) -> "a{sv}":
            return self._metadata

        @dbus_property()
        def Volume(self) -> "d":
            return self._volume

        @Volume.setter
        def Volume(self, value: "d"):
            self._volume = max(0.0, min(1.0, value))
            asyncio.create_task(self._callback("set_volume", int(self._volume * 100)))

        @dbus_property()
        def Position(self) -> "x":
            return int(self._position * 1_000_000)

        @dbus_property()
        def MinimumRate(self) -> "d":
            return 1.0

        @dbus_property()
        def MaximumRate(self) -> "d":
            return 1.0

        @dbus_property()
        def CanGoNext(self) -> "b":
            return True

        @dbus_property()
        def CanGoPrevious(self) -> "b":
            return True

        @dbus_property()
        def CanPlay(self) -> "b":
            return True

        @dbus_property()
        def CanPause(self) -> "b":
            return True

        @dbus_property()
        def CanSeek(self) -> "b":
            return True

        @dbus_property()
        def CanControl(self) -> "b":
            return True

        def set_playing(self, playing: bool):
            self._is_playing = playing

        def set_position(self, pos: float):
            self._position = pos

        def set_volume(self, vol: int):
            self._volume = vol / 100.0

        def set_metadata(self, title: str, artist: str, duration: int, art_url: str, track_id: str):
            self._metadata = {
                "mpris:trackid": Variant("o", f"/org/mpris/MediaPlayer2/Track/{track_id or 'unknown'}"),
                "mpris:length": Variant("x", duration * 1_000_000),
                "xesam:title": Variant("s", title),
                "xesam:artist": Variant("as", [artist] if artist else []),
            }
            if art_url:
                self._metadata["mpris:artUrl"] = Variant("s", art_url)


# ============================================
# Cross-platform MediaKeysHandler
# ============================================

class MediaKeysHandler:
    """Cross-platform media keys handler."""

    def __init__(self, callback: Callable):
        """
        Initialize media keys handler.

        Args:
            callback: Async function called with (command, *args).
                     Commands: 'play', 'pause', 'play_pause', 'stop',
                              'next', 'previous', 'seek', 'set_volume', 'quit'
        """
        self._callback = callback
        self._running = False

        # Platform-specific components
        self._bus: Optional[Any] = None
        self._root_interface: Optional[Any] = None
        self._player_interface: Optional[Any] = None
        self._keyboard_listener: Optional[Any] = None

    @property
    def is_available(self) -> bool:
        """Check if media keys are available on this platform."""
        if IS_LINUX:
            return MPRIS_AVAILABLE
        return PYNPUT_AVAILABLE

    @property
    def is_running(self) -> bool:
        """Check if media keys handler is active."""
        return self._running

    @property
    def backend_name(self) -> str:
        """Get the name of the active backend."""
        if IS_LINUX and MPRIS_AVAILABLE:
            return "MPRIS (D-Bus)"
        elif PYNPUT_AVAILABLE:
            return "pynput (global hotkeys)"
        return "none"

    async def start(self) -> bool:
        """Start listening for media keys."""
        if self._running:
            return True

        if IS_LINUX:
            return await self._start_mpris()
        elif PYNPUT_AVAILABLE:
            return self._start_pynput()

        logger.info("No media key backend available")
        return False

    async def stop(self):
        """Stop listening for media keys."""
        if IS_LINUX and self._bus:
            try:
                self._bus.disconnect()
            except Exception:
                pass

        if self._keyboard_listener:
            try:
                self._keyboard_listener.stop()
            except Exception:
                pass

        self._running = False
        logger.info("Media keys handler stopped")

    async def _start_mpris(self) -> bool:
        """Start MPRIS D-Bus service (Linux)."""
        if not MPRIS_AVAILABLE:
            return False

        try:
            self._bus = await MessageBus(bus_type=BusType.SESSION).connect()
            self._root_interface = MPRISRootInterface(self._callback)
            self._player_interface = MPRISPlayerInterface(self._callback)

            self._bus.export("/org/mpris/MediaPlayer2", self._root_interface)
            self._bus.export("/org/mpris/MediaPlayer2", self._player_interface)

            await self._bus.request_name("org.mpris.MediaPlayer2.wrkmon")

            self._running = True
            logger.info("MPRIS service started - media keys active")
            return True

        except Exception as e:
            logger.warning(f"Failed to start MPRIS: {e}")
            return False

    def _start_pynput(self) -> bool:
        """Start pynput global hotkey listener (Windows/macOS)."""
        if not PYNPUT_AVAILABLE:
            return False

        try:
            def on_press(key):
                try:
                    # Check for media keys
                    if hasattr(key, 'name'):
                        if key == keyboard.Key.media_play_pause:
                            asyncio.create_task(self._callback("play_pause"))
                        elif key == keyboard.Key.media_next:
                            asyncio.create_task(self._callback("next"))
                        elif key == keyboard.Key.media_previous:
                            asyncio.create_task(self._callback("previous"))
                        elif key == keyboard.Key.media_volume_up:
                            asyncio.create_task(self._callback("volume_up"))
                        elif key == keyboard.Key.media_volume_down:
                            asyncio.create_task(self._callback("volume_down"))
                        elif key == keyboard.Key.media_volume_mute:
                            asyncio.create_task(self._callback("mute"))
                except Exception as e:
                    logger.debug(f"Error handling key: {e}")

            self._keyboard_listener = keyboard.Listener(on_press=on_press)
            self._keyboard_listener.start()

            self._running = True
            logger.info("pynput listener started - media keys active")
            return True

        except Exception as e:
            logger.warning(f"Failed to start pynput listener: {e}")
            return False

    def update_playback(self, is_playing: bool = None, position: float = None, volume: int = None):
        """Update playback state (for MPRIS)."""
        if not self._player_interface:
            return
        if is_playing is not None:
            self._player_interface.set_playing(is_playing)
        if position is not None:
            self._player_interface.set_position(position)
        if volume is not None:
            self._player_interface.set_volume(volume)

    def update_track(self, title: str = "", artist: str = "", duration: int = 0,
                     art_url: str = "", track_id: str = ""):
        """Update current track metadata (for MPRIS)."""
        if self._player_interface:
            self._player_interface.set_metadata(title, artist, duration, art_url, track_id)


# Singleton instance
_handler: Optional[MediaKeysHandler] = None


def get_media_keys_handler(callback: Callable = None) -> Optional[MediaKeysHandler]:
    """Get or create the media keys handler."""
    global _handler
    if _handler is None and callback:
        _handler = MediaKeysHandler(callback)
    return _handler



"""Configuration management for wrkmon."""

import os
import sys
from pathlib import Path
from typing import Any

# Use tomllib for Python 3.11+, fall back to tomli
if sys.version_info >= (3, 11):
    import tomllib
else:
    try:
        import tomli as tomllib
    except ImportError:
        tomllib = None


class Config:
    """Manages application configuration."""

    DEFAULT_CONFIG = {
        "general": {
            "volume": 80,
            "shuffle": False,
            "repeat_mode": "none",  # none, one, all
            "show_trending_on_start": True,
        },
        "player": {
            "mpv_path": "mpv",
            "audio_only": True,
            "no_video": True,
        },
        "cache": {
            "url_ttl_hours": 6,
            "max_entries": 1000,
        },
        "ui": {
            "theme": "github_dark",  # github_dark, matrix, dracula, nord
            "show_fake_stats": True,
            "show_thumbnails": True,
            "thumbnail_style": "colored_blocks",  # colored_blocks, colored_simple, braille, blocks
            "thumbnail_width": 45,
        },
    }

    def __init__(self):
        self._config: dict[str, Any] = {}
        self._config_dir = self._get_config_dir()
        self._config_file = self._config_dir / "config.toml"
        self._data_dir = self._get_data_dir()
        self._ensure_dirs()
        self._load()

    def _get_config_dir(self) -> Path:
        """Get the configuration directory path."""
        if sys.platform == "win32":
            base = Path(os.environ.get("APPDATA", Path.home() / "AppData" / "Roaming"))
        else:
            base = Path(os.environ.get("XDG_CONFIG_HOME", Path.home() / ".config"))
        return base / "wrkmon"

    def _get_data_dir(self) -> Path:
        """Get the data directory path."""
        if sys.platform == "win32":
            base = Path(os.environ.get("LOCALAPPDATA", Path.home() / "AppData" / "Local"))
        else:
            base = Path(os.environ.get("XDG_DATA_HOME", Path.home() / ".local" / "share"))
        return base / "wrkmon"

    def _ensure_dirs(self) -> None:
        """Ensure configuration and data directories exist."""
        self._config_dir.mkdir(parents=True, exist_ok=True)
        self._data_dir.mkdir(parents=True, exist_ok=True)

    def _load(self) -> None:
        """Load configuration from file."""
        # Deep copy default config
        import copy
        self._config = copy.deepcopy(self.DEFAULT_CONFIG)

        if self._config_file.exists() and tomllib is not None:
            try:
                with open(self._config_file, "rb") as f:
                    user_config = tomllib.load(f)
                self._merge_config(user_config)
            except Exception:
                pass  # Use defaults on error

    def _merge_config(self, user_config: dict[str, Any]) -> None:
        """Merge user config into default config."""
        for section, values in user_config.items():
            if section in self._config and isinstance(values, dict):
                self._config[section].update(values)
            else:
                self._config[section] = values

    def save(self) -> None:
        """Save current configuration to file."""
        lines = []
        for section, values in self._config.items():
            lines.append(f"[{section}]")
            for key, value in values.items():
                if isinstance(value, bool):
                    lines.append(f"{key} = {str(value).lower()}")
                elif isinstance(value, str):
                    lines.append(f'{key} = "{value}"')
                else:
                    lines.append(f"{key} = {value}")
            lines.append("")

        self._config_file.write_text("\n".join(lines))

    def get(self, section: str, key: str, default: Any = None) -> Any:
        """Get a configuration value."""
        return self._config.get(section, {}).get(key, default)

    def set(self, section: str, key: str, value: Any) -> None:
        """Set a configuration value."""
        if section not in self._config:
            self._config[section] = {}
        self._config[section][key] = value

    @property
    def config_dir(self) -> Path:
        """Get configuration directory path."""
        return self._config_dir

    @property
    def data_dir(self) -> Path:
        """Get data directory path."""
        return self._data_dir

    @property
    def database_path(self) -> Path:
        """Get database file path."""
        return self._data_dir / "wrkmon.db"

    @property
    def cache_path(self) -> Path:
        """Get cache file path."""
        return self._data_dir / "cache.db"

    # General settings
    @property
    def volume(self) -> int:
        """Get current volume setting."""
        return self.get("general", "volume", 80)

    @volume.setter
    def volume(self, value: int) -> None:
        """Set volume."""
        self.set("general", "volume", max(0, min(100, value)))

    @property
    def repeat_mode(self) -> str:
        """Get repeat mode (none, one, all)."""
        return self.get("general", "repeat_mode", "none")

    @repeat_mode.setter
    def repeat_mode(self, value: str) -> None:
        """Set repeat mode."""
        if value in ("none", "one", "all"):
            self.set("general", "repeat_mode", value)

    @property
    def shuffle(self) -> bool:
        """Get shuffle setting."""
        return self.get("general", "shuffle", False)

    @shuffle.setter
    def shuffle(self, value: bool) -> None:
        """Set shuffle."""
        self.set("general", "shuffle", value)

    @property
    def show_trending_on_start(self) -> bool:
        """Whether to show trending videos on startup."""
        return self.get("general", "show_trending_on_start", True)

    @show_trending_on_start.setter
    def show_trending_on_start(self, value: bool) -> None:
        """Set show trending on start."""
        self.set("general", "show_trending_on_start", value)

    # Player settings
    @property
    def mpv_path(self) -> str:
        """Get mpv executable path."""
        from wrkmon.utils.mpv_installer import get_mpv_path
        configured = self.get("player", "mpv_path", "mpv")
        if configured != "mpv":
            return configured
        return get_mpv_path()

    # Cache settings
    @property
    def url_ttl_hours(self) -> int:
        """Get URL cache TTL in hours."""
        return self.get("cache", "url_ttl_hours", 6)

    # UI settings
    @property
    def theme(self) -> str:
        """Get UI theme."""
        return self.get("ui", "theme", "github_dark")

    @theme.setter
    def theme(self, value: str) -> None:
        """Set UI theme."""
        self.set("ui", "theme", value)

    @property
    def show_thumbnails(self) -> bool:
        """Whether to show thumbnails."""
        return self.get("ui", "show_thumbnails", True)

    @show_thumbnails.setter
    def show_thumbnails(self, value: bool) -> None:
        """Set show thumbnails."""
        self.set("ui", "show_thumbnails", value)

    @property
    def thumbnail_style(self) -> str:
        """Get thumbnail rendering style."""
        return self.get("ui", "thumbnail_style", "colored_blocks")

    @thumbnail_style.setter
    def thumbnail_style(self, value: str) -> None:
        """Set thumbnail style."""
        if value in ("colored_blocks", "colored_simple", "braille", "blocks"):
            self.set("ui", "thumbnail_style", value)

    @property
    def thumbnail_width(self) -> int:
        """Get thumbnail width in characters."""
        return self.get("ui", "thumbnail_width", 45)

    @thumbnail_width.setter
    def thumbnail_width(self, value: int) -> None:
        """Set thumbnail width."""
        self.set("ui", "thumbnail_width", max(20, min(80, value)))


# Global config instance
_config: Config | None = None


def get_config() -> Config:
    """Get the global config instance."""
    global _config
    if _config is None:
        _config = Config()
    return _config



"""Theme and CSS for wrkmon TUI."""

# Main application CSS with enhanced UX
APP_CSS = """
/* ============================================
   WRKMON - Terminal Music Player Theme
   Enhanced UX with better colors and visuals
   ============================================ */

/* Base screen styling */
Screen {
    background: #0d1117;
}

/* ----------------------------------------
   HEADER BAR
   ---------------------------------------- */
HeaderBar {
    dock: top;
    height: 1;
    background: #161b22;
    color: #58a6ff;
}

#header-inner {
    width: 100%;
}

#app-title {
    width: auto;
    color: #58a6ff;
    text-style: bold;
    padding: 0 1;
}

#current-view {
    width: 1fr;
    color: #8b949e;
    padding: 0 1;
}

#sys-stats {
    width: auto;
    color: #6e7681;
    padding: 0 1;
}

#update-indicator {
    width: auto;
    color: #f0883e;
    text-style: bold;
    padding: 0 1;
}

#update-indicator.update-available {
    color: #f85149;
    text-style: bold blink;
}

/* ----------------------------------------
   PLAYER BAR (Bottom) - Enhanced
   ---------------------------------------- */
PlayerBar {
    dock: bottom;
    height: 5;
    background: #161b22;
    border-top: solid #30363d;
    padding: 0 1;
}

#player-bar-inner {
    height: 100%;
}

#now-playing-row {
    height: 1;
}

#now-label {
    width: 4;
    color: #8b949e;
}

#play-status {
    width: 3;
    color: #3fb950;
    text-style: bold;
}

#play-status.paused {
    color: #f0883e;
}

#play-status.stopped {
    color: #6e7681;
}

#track-title {
    width: 1fr;
    color: #f0f6fc;
}

#progress-row {
    height: 1;
    padding: 0 1;
}

#time-current {
    width: 8;
    color: #3fb950;
}

#time-total {
    width: 8;
    color: #8b949e;
}

#progress {
    width: 1fr;
    background: #21262d;
}

#progress > .bar--bar {
    color: #238636;
    background: #238636;
}

#progress > .bar--complete {
    color: #3fb950;
}

#volume-row {
    height: 1;
}

#vol-label {
    width: 4;
    color: #8b949e;
}

#volume {
    width: 20;
    background: #21262d;
}

#volume > .bar--bar {
    color: #1f6feb;
    background: #1f6feb;
}

#vol-value {
    width: 5;
    color: #58a6ff;
}

#repeat-indicator {
    width: auto;
    color: #a371f7;
    text-style: bold;
    padding: 0 1;
}

/* ----------------------------------------
   CONTENT SWITCHER / MAIN AREA
   ---------------------------------------- */
#content-area {
    height: 1fr;
}

ContentSwitcher {
    height: 1fr;
}

/* ----------------------------------------
   VIEW CONTAINERS
   ---------------------------------------- */
SearchView, QueueView, HistoryView, PlaylistsView {
    height: 1fr;
    padding: 0 1;
}

#view-title {
    height: 1;
    color: #58a6ff;
    text-style: bold;
    background: #161b22;
    padding: 0 1;
}

/* Search container */
#search-container {
    height: auto;
    padding: 1 0;
}

#search-input {
    width: 100%;
    background: #0d1117;
    border: tall #30363d;
    color: #f0f6fc;
}

#search-input:focus {
    border: tall #58a6ff;
}

#search-input.-invalid {
    border: tall #f85149;
}

/* List headers (column titles) */
#list-header {
    height: 1;
    color: #8b949e;
    background: #161b22;
    text-style: bold;
}

/* Result/Queue/History lists */
#results-list, #queue-list, #history-list, #playlist-list {
    height: 1fr;
    background: #0d1117;
    scrollbar-background: #161b22;
    scrollbar-color: #30363d;
    scrollbar-color-hover: #484f58;
    scrollbar-color-active: #6e7681;
}

ListView > ListItem {
    height: 1;
    padding: 0 1;
    color: #c9d1d9;
}

ListView > ListItem:hover {
    background: #161b22;
}

ListView > ListItem.-highlight, ListView > ListItem:focus {
    background: #1f6feb20;
    color: #58a6ff;
}

/* Track status colors */
.track-playing {
    color: #3fb950;
    text-style: bold;
}

.track-queued {
    color: #f0883e;
}

.track-paused {
    color: #f0883e;
}

.result-text, .queue-text, .history-text {
    width: 100%;
}

/* Status bar - Enhanced */
#status-bar {
    dock: bottom;
    height: 1;
    color: #8b949e;
    background: #161b22;
    padding: 0 1;
}

/* ----------------------------------------
   SEARCH CONTENT & THUMBNAIL
   ---------------------------------------- */
#search-content {
    height: 1fr;
}

#thumbnail-panel {
    width: 50;
    min-width: 50;
    max-width: 50;
    height: 100%;
    background: #0d1117;
    border-left: solid #30363d;
    padding: 0 1;
}

#thumbnail-panel.hidden {
    display: none;
}

#thumb-title {
    height: 1;
    color: #58a6ff;
    text-style: bold;
    background: #161b22;
    padding: 0 1;
}

#thumb-preview {
    height: 1fr;
    background: #0d1117;
    color: #3fb950;
}

#thumb-preview.loading {
    color: #6e7681;
}

#ascii-content {
    height: auto;
    width: 100%;
}

.load-more {
    color: #a371f7;
    text-style: bold;
}

/* ----------------------------------------
   QUEUE VIEW SPECIFICS
   ---------------------------------------- */
#now-playing-section {
    height: auto;
    background: #161b22;
    border: solid #30363d;
    padding: 1;
    margin-bottom: 1;
}

#section-header {
    color: #8b949e;
    text-style: bold;
}

#current-track {
    color: #3fb950;
    padding: 0 1;
}

#playback-progress {
    height: 1;
    padding: 0 1;
}

#track-progress {
    width: 1fr;
    background: #21262d;
}

#track-progress > .bar--bar {
    color: #238636;
    background: #238636;
}

#pos-time, #dur-time {
    width: 8;
    color: #8b949e;
}

#mode-indicators {
    height: 1;
    padding: 0 1;
}

#shuffle-indicator, #repeat-indicator {
    width: auto;
    color: #a371f7;
    padding: 0 1;
}

/* ----------------------------------------
   PLAYLIST INPUT
   ---------------------------------------- */
#new-playlist-input {
    width: 100%;
    background: #0d1117;
    border: tall #30363d;
    color: #f0f6fc;
    margin: 1 0;
}

#new-playlist-input:focus {
    border: tall #58a6ff;
}

/* ----------------------------------------
   HELP OVERLAY
   ---------------------------------------- */
HelpScreen {
    align: center middle;
}

#help-container {
    width: 70;
    height: auto;
    max-height: 80%;
    background: #161b22;
    border: thick #30363d;
    padding: 1 2;
}

#help-title {
    text-align: center;
    color: #58a6ff;
    text-style: bold;
    padding-bottom: 1;
}

#help-content {
    color: #c9d1d9;
}

.help-section {
    color: #f0883e;
    text-style: bold;
    padding-top: 1;
}

.help-key {
    color: #a371f7;
    text-style: bold;
}

.help-desc {
    color: #8b949e;
}

/* ----------------------------------------
   METADATA PANEL
   ---------------------------------------- */
#metadata-panel {
    height: auto;
    background: #161b22;
    border: solid #30363d;
    padding: 1;
    margin-top: 1;
}

#metadata-title {
    color: #f0f6fc;
    text-style: bold;
}

#metadata-channel {
    color: #8b949e;
}

#metadata-stats {
    color: #6e7681;
}

/* ----------------------------------------
   NOTIFICATIONS
   ---------------------------------------- */
Toast {
    background: #161b22;
    border: solid #30363d;
    color: #c9d1d9;
}

Toast.-information {
    border: solid #58a6ff;
}

Toast.-warning {
    border: solid #f0883e;
}

Toast.-error {
    border: solid #f85149;
}

/* ----------------------------------------
   FOOTER - Enhanced
   ---------------------------------------- */
Footer {
    background: #161b22;
    color: #8b949e;
}

Footer > .footer--key {
    color: #58a6ff;
    background: #21262d;
    text-style: bold;
}

Footer > .footer--description {
    color: #8b949e;
}

/* ----------------------------------------
   FOCUS INDICATORS
   ---------------------------------------- */
Input:focus {
    border: tall #58a6ff;
}

Button:focus {
    background: #1f6feb;
}

"""

# Color palette for programmatic access
COLORS = {
    # Base colors (GitHub Dark theme inspired)
    "bg_primary": "#0d1117",
    "bg_secondary": "#161b22",
    "bg_tertiary": "#21262d",
    "border": "#30363d",
    "border_active": "#58a6ff",

    # Text colors
    "text_primary": "#f0f6fc",
    "text_secondary": "#c9d1d9",
    "text_muted": "#8b949e",
    "text_faint": "#6e7681",

    # Accent colors
    "blue": "#58a6ff",
    "green": "#3fb950",
    "green_dark": "#238636",
    "orange": "#f0883e",
    "red": "#f85149",
    "purple": "#a371f7",
    "cyan": "#39c5cf",
    "yellow": "#d29922",

    # Status colors
    "playing": "#3fb950",
    "paused": "#f0883e",
    "stopped": "#6e7681",
    "queued": "#f0883e",
    "error": "#f85149",
    "success": "#3fb950",
    "warning": "#f0883e",
    "info": "#58a6ff",
}

# Alternative themes
THEMES = {
    "github_dark": {
        "primary": "#58a6ff",
        "secondary": "#8b949e",
        "accent": "#a371f7",
        "success": "#3fb950",
        "warning": "#f0883e",
        "error": "#f85149",
        "background": "#0d1117",
        "surface": "#161b22",
    },
    "matrix": {
        "primary": "#00ff00",
        "secondary": "#008800",
        "accent": "#00ffff",
        "success": "#00ff00",
        "warning": "#ffff00",
        "error": "#ff0000",
        "background": "#0a0a0a",
        "surface": "#1a1a1a",
    },
    "dracula": {
        "primary": "#bd93f9",
        "secondary": "#6272a4",
        "accent": "#ff79c6",
        "success": "#50fa7b",
        "warning": "#ffb86c",
        "error": "#ff5555",
        "background": "#282a36",
        "surface": "#44475a",
    },
    "nord": {
        "primary": "#88c0d0",
        "secondary": "#81a1c1",
        "accent": "#b48ead",
        "success": "#a3be8c",
        "warning": "#ebcb8b",
        "error": "#bf616a",
        "background": "#2e3440",
        "surface": "#3b4252",
    },
}



"""Search view container for wrkmon."""

import asyncio
import logging
from textual.app import ComposeResult
from textual.containers import Vertical, Horizontal
from textual.widgets import Static, Input, ListView, ListItem, Label
from textual.binding import Binding
from textual.events import Key
from textual import on

from wrkmon.core.youtube import SearchResult
from wrkmon.ui.messages import TrackSelected, TrackQueued, StatusMessage
from wrkmon.ui.widgets.result_item import ResultItem
from wrkmon.ui.widgets.thumbnail import ThumbnailPreview
from wrkmon.utils.config import get_config

logger = logging.getLogger("wrkmon.search")


class LoadMoreItem(ListItem):
    """Special list item for loading more results."""

    def __init__(self) -> None:
        super().__init__()
        self.is_load_more = True

    def compose(self):
        yield Label("  >>> Load More Results <<<", classes="load-more")


class SearchView(Vertical):
    """Main search view - search YouTube and display results."""

    BINDINGS = [
        Binding("a", "queue_selected", "Add to Queue", show=True),
        Binding("escape", "clear_search", "Clear", show=True),
        Binding("/", "focus_search", "Search", show=True),
        Binding("space", "play_selected", "Play", show=False, priority=True),
        Binding("r", "toggle_repeat", "Repeat", show=True),
        Binding("t", "toggle_thumbnail", "Thumb", show=True),
        Binding("y", "cycle_thumbnail_style", "Style", show=True),
    ]

    def __init__(self, **kwargs) -> None:
        super().__init__(**kwargs)
        self.results: list[SearchResult] = []
        self._is_searching = False
        self._current_query = ""
        self._load_more_offset = 0
        self._batch_size = 15
        self._thumbnail_task: asyncio.Task | None = None
        self._is_trending = False  # True when showing trending, False after search

        # Load settings from config
        self._config = get_config()
        self._show_thumbnail = self._config.show_thumbnails
        self._thumbnail_style = self._config.thumbnail_style
        self._thumbnail_width = self._config.thumbnail_width

    def compose(self) -> ComposeResult:
        yield Static("TRENDING", id="view-title")

        with Vertical(id="search-container"):
            yield Input(
                placeholder="Search YouTube music...",
                id="search-input",
            )

        yield Static(
            " #   Title                                     Channel         Duration  Views",
            id="list-header",
        )

        with Horizontal(id="search-content"):
            yield ListView(id="results-list")
            with Vertical(id="thumbnail-panel"):
                yield Static("PREVIEW", id="thumb-title")
                yield ThumbnailPreview(
                    width=self._thumbnail_width,
                    style=self._thumbnail_style,
                    id="thumb-preview"
                )

        yield Static("Loading trending music...", id="status-bar")

    def on_mount(self) -> None:
        """Load trending videos and setup initial state on mount."""
        # Apply initial thumbnail visibility
        if not self._show_thumbnail:
            try:
                self.query_one("#thumbnail-panel", Vertical).add_class("hidden")
            except Exception:
                pass

        # Load trending videos if enabled
        if self._config.show_trending_on_start:
            asyncio.create_task(self._load_trending())
        else:
            self.query_one("#search-input", Input).focus()
            self._update_status("Type to search YouTube music")

    async def _load_trending(self) -> None:
        """Load trending/popular music videos on startup."""
        self._is_searching = True
        self._is_trending = True
        self._update_status("Loading trending music...")

        try:
            # Update view title to show trending
            self.query_one("#view-title", Static).update("TRENDING")

            # Access the app's YouTube client
            youtube = self.app.youtube
            self.results = await youtube.get_trending_music(max_results=15)

            if self.results:
                self._display_results(show_load_more=False)
                self._update_status(f"Trending: {len(self.results)} popular tracks | Search to find more")
            else:
                self._update_status("Could not load trending | Type to search")

        except Exception as e:
            logger.debug(f"Failed to load trending: {e}")
            self._update_status("Type to search YouTube music")
        finally:
            self._is_searching = False
            # Focus the list if we have results, otherwise search input
            if self.results:
                self.query_one("#results-list", ListView).focus()
            else:
                self.query_one("#search-input", Input).focus()

    @on(Input.Submitted, "#search-input")
    async def handle_search(self, event: Input.Submitted) -> None:
        """Execute search when Enter is pressed."""
        query = event.value.strip()
        if not query or self._is_searching:
            return

        self._is_searching = True
        self._is_trending = False  # No longer showing trending
        self._update_status("Searching...")
        self._current_query = query
        self._load_more_offset = 0

        # Update view title to SEARCH
        try:
            self.query_one("#view-title", Static).update("SEARCH")
        except Exception:
            pass

        try:
            # Access the app's YouTube client
            youtube = self.app.youtube
            self.results = await youtube.search(query, max_results=self._batch_size)
            self._display_results(show_load_more=True)
        except Exception as e:
            self._update_status(f"Search failed: {e}")
            self.post_message(StatusMessage(f"Search error: {e}", "error"))
        finally:
            self._is_searching = False

    async def _load_more_results(self) -> None:
        """Load more search results."""
        if not self._current_query or self._is_searching:
            return

        self._is_searching = True
        self._update_status("Loading more...")

        try:
            youtube = self.app.youtube
            self._load_more_offset += self._batch_size
            # Search with offset by fetching more and skipping existing
            new_results = await youtube.search(
                self._current_query,
                max_results=self._batch_size + self._load_more_offset
            )
            # Get only the new results we don't have yet
            if len(new_results) > len(self.results):
                self.results = new_results
                self._display_results(show_load_more=True)
            else:
                self._update_status(f"No more results | Found {len(self.results)} total")
        except Exception as e:
            self._update_status(f"Load more failed: {e}")
        finally:
            self._is_searching = False

    def _display_results(self, show_load_more: bool = False) -> None:
        """Display search results in the list."""
        list_view = self.query_one("#results-list", ListView)
        list_view.clear()

        if not self.results:
            self._update_status("No results found")
            return

        for i, result in enumerate(self.results, 1):
            list_view.append(ResultItem(result, i))

        # Add "Load More" option at the end
        if show_load_more:
            list_view.append(LoadMoreItem())

        # Build status with repeat indicator
        repeat_status = self._get_repeat_status()
        thumb_status = " [THUMB ON]" if self._show_thumbnail else ""
        status = f"Found {len(self.results)} | Enter=Play A=Queue R=Repeat T=Thumb{repeat_status}{thumb_status}"
        self._update_status(status)
        list_view.focus()

    def _get_repeat_status(self) -> str:
        """Get repeat mode status string."""
        try:
            mode = self.app.queue.repeat_mode
            if mode == "one":
                return " [R1]"
            elif mode == "all":
                return " [RA]"
        except Exception:
            pass
        return ""

    def _update_status(self, message: str) -> None:
        """Update the status bar."""
        try:
            self.query_one("#status-bar", Static).update(message)
        except Exception:
            pass

    def _get_selected(self) -> SearchResult | None:
        """Get the currently selected result."""
        list_view = self.query_one("#results-list", ListView)
        if list_view.highlighted_child is None:
            return None

        item = list_view.highlighted_child
        if isinstance(item, ResultItem):
            return item.result
        return None

    def _update_thumbnail_preview(self) -> None:
        """Update thumbnail preview for currently highlighted item."""
        if not self._show_thumbnail:
            return

        result = self._get_selected()
        if result:
            try:
                preview = self.query_one("#thumb-preview", ThumbnailPreview)
                title = self.query_one("#thumb-title", Static)
                title.update(result.title[:35] + "..." if len(result.title) > 35 else result.title)
                preview.set_video(result.video_id)
            except Exception:
                pass
        else:
            self._clear_thumbnail()

    def _clear_thumbnail(self) -> None:
        """Clear the thumbnail preview."""
        try:
            preview = self.query_one("#thumb-preview", ThumbnailPreview)
            title = self.query_one("#thumb-title", Static)
            title.update("PREVIEW")
            preview.clear()
        except Exception:
            pass

    @on(ListView.Highlighted, "#results-list")
    def handle_highlight_changed(self, event: ListView.Highlighted) -> None:
        """Update thumbnail when highlighted item changes."""
        self._update_thumbnail_preview()

    def action_toggle_thumbnail(self) -> None:
        """Toggle thumbnail preview visibility."""
        self._show_thumbnail = not self._show_thumbnail
        # Save to config
        self._config.show_thumbnails = self._show_thumbnail

        try:
            panel = self.query_one("#thumbnail-panel", Vertical)
            if self._show_thumbnail:
                panel.remove_class("hidden")
                self._update_thumbnail_preview()
            else:
                panel.add_class("hidden")

            # Update status
            repeat_status = self._get_repeat_status()
            thumb_status = " [THUMB ON]" if self._show_thumbnail else ""
            count = len(self.results) if self.results else 0
            self._update_status(f"Found {count} | T=Thumb{repeat_status}{thumb_status}")
        except Exception:
            pass

    def action_cycle_thumbnail_style(self) -> None:
        """Cycle through thumbnail rendering styles (Y key)."""
        styles = ["colored_blocks", "colored_simple", "braille", "blocks"]
        style_names = {
            "colored_blocks": "Color Blocks",
            "colored_simple": "Color Simple",
            "braille": "Braille",
            "blocks": "Grayscale",
        }

        try:
            current_idx = styles.index(self._thumbnail_style) if self._thumbnail_style in styles else 0
            next_idx = (current_idx + 1) % len(styles)
            self._thumbnail_style = styles[next_idx]

            # Save to config
            self._config.thumbnail_style = self._thumbnail_style

            # Update the preview widget
            preview = self.query_one("#thumb-preview", ThumbnailPreview)
            preview.set_style(self._thumbnail_style)

            # Update status to show new style
            self._update_status(f"Thumbnail style: {style_names[self._thumbnail_style]}")
        except Exception:
            pass

    def action_clear_search(self) -> None:
        """Clear the search input."""
        search_input = self.query_one("#search-input", Input)
        if search_input.value:
            search_input.value = ""
        self._clear_thumbnail()
        search_input.focus()

    @on(ListView.Selected, "#results-list")
    async def handle_result_selected(self, event: ListView.Selected) -> None:
        """Handle Enter key on a result - play it or load more."""
        if isinstance(event.item, LoadMoreItem):
            await self._load_more_results()
        elif isinstance(event.item, ResultItem):
            result = event.item.result
            self.post_message(TrackSelected(result))
            repeat_status = self._get_repeat_status()
            self._update_status(f"Playing: {result.title[:40]}...{repeat_status}")

    def action_play_selected(self) -> None:
        """Play the selected track (Space key)."""
        list_view = self.query_one("#results-list", ListView)
        if not list_view.has_focus:
            return
        result = self._get_selected()
        if result:
            self.post_message(TrackSelected(result))
            repeat_status = self._get_repeat_status()
            self._update_status(f"Playing: {result.title[:40]}...{repeat_status}")

    def action_toggle_repeat(self) -> None:
        """Cycle repeat mode (R key)."""
        try:
            mode = self.app.queue.cycle_repeat()
            mode_names = {"none": "OFF", "one": "ONE", "all": "ALL"}
            repeat_status = self._get_repeat_status()
            count = len(self.results) if self.results else 0
            thumb_status = " [THUMB ON]" if self._show_thumbnail else ""
            self._update_status(f"Repeat: {mode_names[mode]} | Found {count}{repeat_status}{thumb_status}")
        except Exception:
            pass

    def action_queue_selected(self) -> None:
        """Add selected track to queue."""
        logger.info("=== 'a' PRESSED: action_queue_selected ===")
        result = self._get_selected()
        logger.info(f"  Selected result: {result}")
        if result:
            logger.info(f"  Posting TrackQueued for: {result.title}")
            self.post_message(TrackQueued(result))
            repeat_status = self._get_repeat_status()
            self._update_status(f"Queued: {result.title[:40]}...{repeat_status}")
        else:
            logger.warning("  No result selected!")

    def focus_input(self) -> None:
        """Focus the search input (called from parent)."""
        self.query_one("#search-input", Input).focus()

    def action_focus_search(self) -> None:
        """Focus the search input (/ key)."""
        self.focus_input()

    def on_key(self, event: Key) -> None:
        """Handle key events - up at top of list goes to search, down from search goes to list."""
        if event.key == "up":
            list_view = self.query_one("#results-list", ListView)
            # If list is focused and at top (index 0), go to search input
            if list_view.has_focus and list_view.index == 0:
                self.focus_input()
                event.prevent_default()
                event.stop()
        elif event.key == "down":
            search_input = self.query_one("#search-input", Input)
            # If search input is focused, go to list
            if search_input.has_focus:
                list_view = self.query_one("#results-list", ListView)
                if len(list_view.children) > 0:
                    list_view.focus()
                    list_view.index = 0
                    event.prevent_default()
                    event.stop()

    def focus_list(self) -> None:
        """Focus the results list (called from parent)."""
        list_view = self.query_one("#results-list", ListView)
        if len(list_view.children) > 0:
            list_view.focus()
        else:
            self.query_one("#search-input", Input).focus()
